#pragma once

// @generated by aten/src/ATen/gen.py

#include <ATen/Context.h>
#include <c10/core/ScalarType.h>
#include <ATen/core/TensorMethods.h>
#include <ATen/core/TensorOptions.h>

#include <array>
#include <functional>
#include <string>
#include <tuple>
#include <vector>

namespace c10 {
class Scalar;
}
namespace at {
struct Generator;
class Tensor;
struct Type;
} // namespace at

namespace at {
namespace native {

inline Tensor from_blob(
    void* data,
    IntList sizes,
    const std::function<void(void*)>& deleter,
    const TensorOptions& options = {}) {
  return at::getType(options).tensorFromBlob(data, sizes, deleter);
}

inline Tensor from_blob(
    void* data,
    IntList sizes,
    IntList strides,
    const std::function<void(void*)>& deleter,
    const TensorOptions& options = {}) {
  return at::getType(options).tensorFromBlob(data, sizes, strides, deleter);
}

// These functions are defined in native/TensorFactories.cpp.
#define TENSOR(T, S, _1)                                                      \
  CAFFE2_API Tensor tensor(ArrayRef<T> values, const TensorOptions& options); \
  inline Tensor tensor(                                                       \
      std::initializer_list<T> values, const TensorOptions& options) {        \
    return native::tensor(ArrayRef<T>(values), options);                      \
  }                                                                           \
  inline Tensor tensor(T value, const TensorOptions& options) {               \
    return native::tensor(ArrayRef<T>(value), options);                       \
  }                                                                           \
  inline Tensor tensor(ArrayRef<T> values) {                                  \
    return native::tensor(std::move(values), at::dtype(k##S));                \
  }                                                                           \
  inline Tensor tensor(std::initializer_list<T> values) {                     \
    return native::tensor(ArrayRef<T>(values));                               \
  }                                                                           \
  inline Tensor tensor(T value) {                                             \
    return native::tensor(ArrayRef<T>(value));                                \
  }
AT_FORALL_SCALAR_TYPES_EXCEPT_HALF(TENSOR)
#undef TENSOR

CAFFE2_API Tensor _cast_Byte(const Tensor & self, bool non_blocking=false);
CAFFE2_API Tensor _cast_Char(const Tensor & self, bool non_blocking=false);
CAFFE2_API Tensor _cast_Double(const Tensor & self, bool non_blocking=false);
CAFFE2_API Tensor _cast_Float(const Tensor & self, bool non_blocking=false);
CAFFE2_API Tensor _cast_Int(const Tensor & self, bool non_blocking=false);
CAFFE2_API Tensor _cast_Long(const Tensor & self, bool non_blocking=false);
CAFFE2_API Tensor _cast_Short(const Tensor & self, bool non_blocking=false);
CAFFE2_API Tensor _cast_Half(const Tensor & self, bool non_blocking=false);
CAFFE2_API std::tuple<Tensor,Tensor> _cudnn_ctc_loss(const Tensor & log_probs, const Tensor & targets, IntList input_lengths, IntList target_lengths, int64_t blank, bool deterministic);
CAFFE2_API Tensor _cudnn_rnn_flatten_weight(TensorList weight_arr, int64_t weight_stride0, int64_t input_size, int64_t mode, int64_t hidden_size, int64_t num_layers, bool batch_first, bool bidirectional);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor,Tensor,Tensor> _cudnn_rnn(const Tensor & input, TensorList weight, int64_t weight_stride0, const Tensor & weight_buf, const Tensor & hx, const Tensor & cx, int64_t mode, int64_t hidden_size, int64_t num_layers, bool batch_first, double dropout, bool train, bool bidirectional, IntList batch_sizes, const Tensor & dropout_state);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor,std::vector<Tensor>> _cudnn_rnn_backward(const Tensor & input, TensorList weight, int64_t weight_stride0, const Tensor & weight_buf, const Tensor & hx, const Tensor & cx, const Tensor & output, const Tensor & grad_output, const Tensor & grad_hy, const Tensor & grad_cy, int64_t mode, int64_t hidden_size, int64_t num_layers, bool batch_first, double dropout, bool train, bool bidirectional, IntList batch_sizes, const Tensor & dropout_state, const Tensor & reserve, std::array<bool,4> output_mask);
CAFFE2_API Tensor _cudnn_init_dropout_state(double dropout, bool train, int64_t dropout_seed, const TensorOptions & options);
CAFFE2_API std::tuple<Tensor,Tensor> fused_dropout_cuda(const Tensor & self, double p, Generator * generator=nullptr);
CAFFE2_API Tensor masked_scale_cuda(const Tensor & self, const Tensor & mask, double scale);
CAFFE2_API Tensor _reshape_from_tensor(const Tensor & self, const Tensor & shape);
CAFFE2_API Tensor _shape_as_tensor(const Tensor & self);
CAFFE2_API Tensor dropout(const Tensor & input, double p, bool train);
CAFFE2_API Tensor & dropout_(Tensor & self, double p, bool train);
CAFFE2_API Tensor feature_dropout(const Tensor & input, double p, bool train);
CAFFE2_API Tensor & feature_dropout_(Tensor & self, double p, bool train);
CAFFE2_API Tensor alpha_dropout(const Tensor & input, double p, bool train);
CAFFE2_API Tensor & alpha_dropout_(Tensor & self, double p, bool train);
CAFFE2_API Tensor feature_alpha_dropout(const Tensor & input, double p, bool train);
CAFFE2_API Tensor & feature_alpha_dropout_(Tensor & self, double p, bool train);
CAFFE2_API Tensor abs(const Tensor & self);
CAFFE2_API Tensor & _abs__cpu(Tensor & self);
CAFFE2_API Tensor & _abs__cuda(Tensor & self);
CAFFE2_API Tensor & _abs_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _abs_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor acos(const Tensor & self);
CAFFE2_API Tensor & _acos__cpu(Tensor & self);
CAFFE2_API Tensor & _acos__cuda(Tensor & self);
CAFFE2_API Tensor & _acos_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _acos_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor avg_pool1d(const Tensor & self, IntList kernel_size, IntList stride={}, IntList padding=0, bool ceil_mode=false, bool count_include_pad=true);
CAFFE2_API Tensor adaptive_avg_pool1d(const Tensor & self, IntList output_size);
CAFFE2_API std::tuple<Tensor,Tensor> adaptive_max_pool1d(const Tensor & self, IntList output_size);
CAFFE2_API Tensor add(const Tensor & self, const Tensor & other, Scalar alpha=1);
CAFFE2_API Tensor & add_(Tensor & self, const Tensor & other, Scalar alpha=1);
CAFFE2_API Tensor & add_out(Tensor & result, const Tensor & self, const Tensor & other, Scalar alpha=1);
CAFFE2_API Tensor add(const Tensor & self, Scalar other, Scalar alpha=1);
CAFFE2_API Tensor & add_(Tensor & self, Scalar other, Scalar alpha=1);
CAFFE2_API Tensor addmv(const Tensor & self, const Tensor & mat, const Tensor & vec, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & addmv_(Tensor & self, const Tensor & mat, const Tensor & vec, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & addmv_out(Tensor & result, const Tensor & self, const Tensor & mat, const Tensor & vec, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor addr(const Tensor & self, const Tensor & vec1, const Tensor & vec2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & addr_(Tensor & self, const Tensor & vec1, const Tensor & vec2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & addr_out(Tensor & result, const Tensor & self, const Tensor & vec1, const Tensor & vec2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor affine_grid_generator(const Tensor & theta, IntList size);
CAFFE2_API Tensor affine_grid_generator_backward(const Tensor & grad, IntList size);
CAFFE2_API Tensor all(const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor & all_out(Tensor & result, const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API bool allclose(const Tensor & self, const Tensor & other, double rtol=1e-05, double atol=1e-08, bool equal_nan=false);
CAFFE2_API Tensor any(const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor & any_out(Tensor & result, const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor arange(Scalar start, Scalar end, const TensorOptions & options={});
CAFFE2_API Tensor arange(Scalar start, Scalar end, Scalar step, const TensorOptions & options={});
CAFFE2_API Tensor & arange_out(Tensor & result, Scalar start, Scalar end);
CAFFE2_API Tensor & arange_out(Tensor & result, Scalar start, Scalar end, Scalar step);
CAFFE2_API Tensor arange(Scalar end, const TensorOptions & options={});
CAFFE2_API Tensor & arange_out(Tensor & result, Scalar end);
CAFFE2_API Tensor _dim_arange(const Tensor & like, int64_t dim);
CAFFE2_API Tensor argmax(const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor argmax(const Tensor & self);
CAFFE2_API Tensor _argmax(const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor argmin(const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor argmin(const Tensor & self);
CAFFE2_API Tensor _argmin(const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor as_strided(const Tensor & self, IntList size, IntList stride);
CAFFE2_API Tensor & as_strided_(Tensor & self, IntList size, IntList stride);
CAFFE2_API Tensor as_strided(const Tensor & self, IntList size, IntList stride, int64_t storage_offset);
CAFFE2_API Tensor & as_strided_(Tensor & self, IntList size, IntList stride, int64_t storage_offset);
CAFFE2_API Tensor asin(const Tensor & self);
CAFFE2_API Tensor & _asin__cpu(Tensor & self);
CAFFE2_API Tensor & _asin__cuda(Tensor & self);
CAFFE2_API Tensor & _asin_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _asin_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor atan(const Tensor & self);
CAFFE2_API Tensor & _atan__cpu(Tensor & self);
CAFFE2_API Tensor & _atan__cuda(Tensor & self);
CAFFE2_API Tensor & _atan_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _atan_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor baddbmm_cpu(const Tensor & self, const Tensor & batch1, const Tensor & batch2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor baddbmm_cuda(const Tensor & self, const Tensor & batch1, const Tensor & batch2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & baddbmm__cpu(Tensor & self, const Tensor & batch1, const Tensor & batch2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & baddbmm__cuda(Tensor & self, const Tensor & batch1, const Tensor & batch2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & _baddbmm_mkl_(Tensor & self, const Tensor & batch1, const Tensor & batch2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & baddbmm_out_cpu(Tensor & result, const Tensor & self, const Tensor & batch1, const Tensor & batch2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & baddbmm_out_cuda(Tensor & result, const Tensor & self, const Tensor & batch1, const Tensor & batch2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor bartlett_window(int64_t window_length, const TensorOptions & options={});
CAFFE2_API Tensor bartlett_window(int64_t window_length, bool periodic, const TensorOptions & options={});
CAFFE2_API Tensor batch_norm(const Tensor & input, const Tensor & weight, const Tensor & bias, const Tensor & running_mean, const Tensor & running_var, bool training, double momentum, double eps, bool cudnn_enabled);
CAFFE2_API Tensor bernoulli(const Tensor & self, Generator * generator=nullptr);
CAFFE2_API Tensor & bernoulli_out(Tensor & result, const Tensor & self, Generator * generator=nullptr);
CAFFE2_API Tensor & bernoulli_tensor_cpu_(Tensor & self, const Tensor & p, Generator * generator=nullptr);
CAFFE2_API Tensor & bernoulli_tensor_cuda_(Tensor & self, const Tensor & p, Generator * generator=nullptr);
CAFFE2_API Tensor & bernoulli_scalar_cpu_(Tensor & self, double p=0.5, Generator * generator=nullptr);
CAFFE2_API Tensor & bernoulli_scalar_cuda_(Tensor & self, double p=0.5, Generator * generator=nullptr);
CAFFE2_API Tensor bernoulli(const Tensor & self, double p, Generator * generator=nullptr);
CAFFE2_API Tensor bilinear(const Tensor & input1, const Tensor & input2, const Tensor & weight, const Tensor & bias);
CAFFE2_API Tensor binary_cross_entropy_with_logits(const Tensor & self, const Tensor & target, const Tensor & weight, const Tensor & pos_weight, int64_t reduction);
CAFFE2_API Tensor binary_cross_entropy_with_logits_backward(const Tensor & grad_output, const Tensor & self, const Tensor & target, const Tensor & weight, const Tensor & pos_weight, int64_t reduction);
CAFFE2_API Tensor _bincount_cpu(const Tensor & self, const Tensor & weights={}, int64_t minlength=0);
CAFFE2_API Tensor _bincount_cuda(const Tensor & self, const Tensor & weights={}, int64_t minlength=0);
CAFFE2_API Tensor blackman_window(int64_t window_length, const TensorOptions & options={});
CAFFE2_API Tensor blackman_window(int64_t window_length, bool periodic, const TensorOptions & options={});
CAFFE2_API Tensor bmm_cpu(const Tensor & self, const Tensor & mat2);
CAFFE2_API Tensor bmm_cuda(const Tensor & self, const Tensor & mat2);
CAFFE2_API Tensor & bmm_out_cpu(Tensor & result, const Tensor & self, const Tensor & mat2);
CAFFE2_API Tensor & bmm_out_cuda(Tensor & result, const Tensor & self, const Tensor & mat2);
CAFFE2_API std::vector<Tensor> broadcast_tensors(TensorList tensors);
CAFFE2_API Tensor cat(TensorList tensors, int64_t dim=0);
CAFFE2_API Tensor & cat_out(Tensor & result, TensorList tensors, int64_t dim=0);
CAFFE2_API Tensor ceil(const Tensor & self);
CAFFE2_API Tensor & _ceil__cpu(Tensor & self);
CAFFE2_API Tensor & _ceil__cuda(Tensor & self);
CAFFE2_API Tensor & _ceil_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _ceil_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor chain_matmul(TensorList matrices);
CAFFE2_API std::vector<Tensor> chunk(const Tensor & self, int64_t chunks, int64_t dim=0);
CAFFE2_API Tensor clamp(const Tensor & self, c10::optional<Scalar> min=c10::nullopt, c10::optional<Scalar> max=c10::nullopt);
CAFFE2_API Tensor & _clamp__cpu(Tensor & self, c10::optional<Scalar> min=c10::nullopt, c10::optional<Scalar> max=c10::nullopt);
CAFFE2_API Tensor & _clamp__cuda(Tensor & self, c10::optional<Scalar> min=c10::nullopt, c10::optional<Scalar> max=c10::nullopt);
CAFFE2_API Tensor & _clamp_out_cpu(Tensor & result, const Tensor & self, c10::optional<Scalar> min=c10::nullopt, c10::optional<Scalar> max=c10::nullopt);
CAFFE2_API Tensor & _clamp_out_cuda(Tensor & result, const Tensor & self, c10::optional<Scalar> min=c10::nullopt, c10::optional<Scalar> max=c10::nullopt);
CAFFE2_API Tensor clamp_max(const Tensor & self, Scalar max);
CAFFE2_API Tensor & _clamp_max__cpu(Tensor & self, Scalar max);
CAFFE2_API Tensor & _clamp_max__cuda(Tensor & self, Scalar max);
CAFFE2_API Tensor & _clamp_max_out_cpu(Tensor & result, const Tensor & self, Scalar max);
CAFFE2_API Tensor & _clamp_max_out_cuda(Tensor & result, const Tensor & self, Scalar max);
CAFFE2_API Tensor clamp_min(const Tensor & self, Scalar min);
CAFFE2_API Tensor & _clamp_min__cpu(Tensor & self, Scalar min);
CAFFE2_API Tensor & _clamp_min__cuda(Tensor & self, Scalar min);
CAFFE2_API Tensor & _clamp_min_out_cpu(Tensor & result, const Tensor & self, Scalar min);
CAFFE2_API Tensor & _clamp_min_out_cuda(Tensor & result, const Tensor & self, Scalar min);
CAFFE2_API bool cudnn_is_acceptable(const Tensor & self);
CAFFE2_API Tensor constant_pad_nd(const Tensor & self, IntList pad, Scalar value=0);
CAFFE2_API Tensor contiguous(const Tensor & self);
CAFFE2_API Tensor convolution(const Tensor & input, const Tensor & weight, const Tensor & bias, IntList stride, IntList padding, IntList dilation, bool transposed, IntList output_padding, int64_t groups);
CAFFE2_API Tensor _convolution(const Tensor & input, const Tensor & weight, const Tensor & bias, IntList stride, IntList padding, IntList dilation, bool transposed, IntList output_padding, int64_t groups, bool benchmark, bool deterministic, bool cudnn_enabled);
CAFFE2_API Tensor _convolution_nogroup(const Tensor & input, const Tensor & weight, const Tensor & bias, IntList stride, IntList padding, IntList dilation, bool transposed, IntList output_padding);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> _convolution_double_backward(const Tensor & ggI, const Tensor & ggW, const Tensor & ggb, const Tensor & gO, const Tensor & weight, const Tensor & self, IntList stride, IntList padding, IntList dilation, bool transposed, IntList output_padding, int64_t groups, bool benchmark, bool deterministic, bool cudnn_enabled, std::array<bool,3> output_mask);
CAFFE2_API Tensor conv1d(const Tensor & input, const Tensor & weight, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList dilation=1, int64_t groups=1);
CAFFE2_API Tensor conv2d(const Tensor & input, const Tensor & weight, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList dilation=1, int64_t groups=1);
CAFFE2_API Tensor conv3d(const Tensor & input, const Tensor & weight, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList dilation=1, int64_t groups=1);
CAFFE2_API Tensor conv_tbc(const Tensor & self, const Tensor & weight, const Tensor & bias, int64_t pad=0);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> conv_tbc_backward(const Tensor & self, const Tensor & input, const Tensor & weight, const Tensor & bias, int64_t pad);
CAFFE2_API Tensor conv_transpose1d(const Tensor & input, const Tensor & weight, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList output_padding=0, int64_t groups=1, IntList dilation=1);
CAFFE2_API Tensor conv_transpose2d(const Tensor & input, const Tensor & weight, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList output_padding=0, int64_t groups=1, IntList dilation=1);
CAFFE2_API Tensor conv_transpose3d(const Tensor & input, const Tensor & weight, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList output_padding=0, int64_t groups=1, IntList dilation=1);
CAFFE2_API Tensor & _s_copy__cpu(Tensor & self, const Tensor & src, bool non_blocking=false);
CAFFE2_API Tensor & _s_copy__cuda(Tensor & self, const Tensor & src, bool non_blocking=false);
CAFFE2_API Tensor _s_copy_from_cuda(const Tensor & self, const Tensor & dst, bool non_blocking=false);
CAFFE2_API void _copy_same_type__cpu(Tensor & self, const Tensor & src);
CAFFE2_API Tensor cos(const Tensor & self);
CAFFE2_API Tensor & _cos__cpu(Tensor & self);
CAFFE2_API Tensor & _cos__cuda(Tensor & self);
CAFFE2_API Tensor & _cos_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _cos_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor cosh(const Tensor & self);
CAFFE2_API Tensor & _cosh__cpu(Tensor & self);
CAFFE2_API Tensor & _cosh__cuda(Tensor & self);
CAFFE2_API Tensor & _cosh_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _cosh_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor cosine_embedding_loss(const Tensor & input1, const Tensor & input2, const Tensor & target, double margin=0.0, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor cudnn_affine_grid_generator_forward(const Tensor & theta, int64_t N, int64_t C, int64_t H, int64_t W);
CAFFE2_API Tensor cudnn_affine_grid_generator_backward(const Tensor & grad, int64_t N, int64_t C, int64_t H, int64_t W);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> cudnn_batch_norm(const Tensor & input, const Tensor & weight, const Tensor & bias, const Tensor & running_mean, const Tensor & running_var, bool training, double exponential_average_factor, double epsilon);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> cudnn_batch_norm_backward(const Tensor & input, const Tensor & grad_output, const Tensor & weight, const Tensor & running_mean, const Tensor & running_var, const Tensor & save_mean, const Tensor & save_var, double epsilon);
CAFFE2_API Tensor cudnn_convolution(const Tensor & self, const Tensor & weight, const Tensor & bias, IntList padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic);
CAFFE2_API Tensor cudnn_convolution_backward_input(IntList self_size, const Tensor & grad_output, const Tensor & weight, IntList padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> cudnn_convolution_backward(const Tensor & self, const Tensor & grad_output, const Tensor & weight, IntList padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic, std::array<bool,3> output_mask);
CAFFE2_API Tensor cudnn_convolution_backward_bias(const Tensor & grad_output);
CAFFE2_API Tensor cudnn_convolution_backward_weight(IntList weight_size, const Tensor & grad_output, const Tensor & self, IntList padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic);
CAFFE2_API Tensor cudnn_convolution_transpose(const Tensor & self, const Tensor & weight, const Tensor & bias, IntList padding, IntList output_padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> cudnn_convolution_transpose_backward(const Tensor & self, const Tensor & grad_output, const Tensor & weight, IntList padding, IntList output_padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic, std::array<bool,3> output_mask);
CAFFE2_API Tensor cudnn_convolution_backward_bias(const Tensor & grad_output);
CAFFE2_API Tensor cudnn_convolution_transpose_backward_input(const Tensor & grad_output, const Tensor & weight, IntList padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic);
CAFFE2_API Tensor cudnn_convolution_transpose_backward_weight(IntList weight_size, const Tensor & grad_output, const Tensor & self, IntList padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic);
CAFFE2_API Tensor cudnn_grid_sampler_forward(const Tensor & self, const Tensor & grid);
CAFFE2_API std::tuple<Tensor,Tensor> cudnn_grid_sampler_backward(const Tensor & self, const Tensor & grid, const Tensor & grad_output);
CAFFE2_API Tensor cumsum(const Tensor & self, int64_t dim, ScalarType dtype);
CAFFE2_API Tensor cumsum(const Tensor & self, int64_t dim);
CAFFE2_API Tensor & cumsum_out(Tensor & result, const Tensor & self, int64_t dim, ScalarType dtype);
CAFFE2_API Tensor & cumsum_out(Tensor & result, const Tensor & self, int64_t dim);
CAFFE2_API Tensor cumprod(const Tensor & self, int64_t dim, ScalarType dtype);
CAFFE2_API Tensor cumprod(const Tensor & self, int64_t dim);
CAFFE2_API Tensor & cumprod_out(Tensor & result, const Tensor & self, int64_t dim, ScalarType dtype);
CAFFE2_API Tensor & cumprod_out(Tensor & result, const Tensor & self, int64_t dim);
CAFFE2_API Tensor ctc_loss(const Tensor & log_probs, const Tensor & targets, IntList input_lengths, IntList target_lengths, int64_t blank=0, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor ctc_loss(const Tensor & log_probs, const Tensor & targets, const Tensor & input_lengths, const Tensor & target_lengths, int64_t blank=0, int64_t reduction=Reduction::Mean);
CAFFE2_API std::tuple<Tensor,Tensor> ctc_loss_cpu(const Tensor & log_probs, const Tensor & targets, IntList input_lengths, IntList target_lengths, int64_t blank=0);
CAFFE2_API std::tuple<Tensor,Tensor> ctc_loss_gpu(const Tensor & log_probs, const Tensor & targets, IntList input_lengths, IntList target_lengths, int64_t blank=0);
CAFFE2_API Tensor ctc_loss_backward_cpu(const Tensor & grad, const Tensor & log_probs, const Tensor & targets, IntList input_lengths, IntList target_lengths, const Tensor & neg_log_likelihood, const Tensor & log_alpha, int64_t blank);
CAFFE2_API Tensor ctc_loss_backward_gpu(const Tensor & grad, const Tensor & log_probs, const Tensor & targets, IntList input_lengths, IntList target_lengths, const Tensor & neg_log_likelihood, const Tensor & log_alpha, int64_t blank);
CAFFE2_API Tensor det(const Tensor & self);
CAFFE2_API Tensor diag_embed(const Tensor & self, int64_t offset=0, int64_t dim1=-2, int64_t dim2=-1);
CAFFE2_API Tensor diagflat(const Tensor & self, int64_t offset=0);
CAFFE2_API Tensor diagonal(const Tensor & self, int64_t offset=0, int64_t dim1=0, int64_t dim2=1);
CAFFE2_API Tensor div(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & div_(Tensor & self, const Tensor & other);
CAFFE2_API Tensor & div_out(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor div(const Tensor & self, Scalar other);
CAFFE2_API Tensor & div_(Tensor & self, Scalar other);
CAFFE2_API Tensor dot(const Tensor & self, const Tensor & tensor);
CAFFE2_API Tensor & dot_out(Tensor & result, const Tensor & self, const Tensor & tensor);
CAFFE2_API Tensor einsum(std::string equation, TensorList tensors);
CAFFE2_API Tensor embedding(const Tensor & weight, const Tensor & indices, int64_t padding_idx=-1, bool scale_grad_by_freq=false, bool sparse=false);
CAFFE2_API Tensor embedding_backward(const Tensor & grad, const Tensor & indices, int64_t num_weights, int64_t padding_idx, bool scale_grad_by_freq, bool sparse);
CAFFE2_API Tensor embedding_dense_backward_cpu(const Tensor & grad, const Tensor & indices, int64_t num_weights, int64_t padding_idx, bool scale_grad_by_freq);
CAFFE2_API Tensor embedding_dense_backward_cuda(const Tensor & grad, const Tensor & indices, int64_t num_weights, int64_t padding_idx, bool scale_grad_by_freq);
CAFFE2_API Tensor & embedding_renorm_cpu_(Tensor & self, const Tensor & indices, double max_norm, double norm_type);
CAFFE2_API Tensor & embedding_renorm_cuda_(Tensor & self, const Tensor & indices, double max_norm, double norm_type);
CAFFE2_API Tensor embedding_sparse_backward(const Tensor & grad, const Tensor & indices, int64_t num_weights, int64_t padding_idx, bool scale_grad_by_freq);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor,Tensor> embedding_bag(const Tensor & weight, const Tensor & indices, const Tensor & offsets, bool scale_grad_by_freq=false, int64_t mode=0, bool sparse=false);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor,Tensor> _embedding_bag_cpu(const Tensor & weight, const Tensor & indices, const Tensor & offsets, bool scale_grad_by_freq=false, int64_t mode=0, bool sparse=false);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor,Tensor> _embedding_bag_cuda(const Tensor & weight, const Tensor & indices, const Tensor & offsets, bool scale_grad_by_freq=false, int64_t mode=0, bool sparse=false);
CAFFE2_API Tensor _embedding_bag_backward(const Tensor & grad, const Tensor & indices, const Tensor & offsets, const Tensor & offset2bag, const Tensor & bag_size, const Tensor & maximum_indices, int64_t num_weights, bool scale_grad_by_freq, int64_t mode, bool sparse);
CAFFE2_API Tensor _embedding_bag_sparse_backward(const Tensor & grad, const Tensor & indices, const Tensor & offsets, const Tensor & offset2bag, const Tensor & bag_size, int64_t num_weights, bool scale_grad_by_freq, int64_t mode);
CAFFE2_API Tensor _embedding_bag_dense_backward_cpu(const Tensor & grad, const Tensor & indices, const Tensor & offsets, const Tensor & offset2bag, const Tensor & bag_size, const Tensor & maximum_indices, int64_t num_weights, bool scale_grad_by_freq, int64_t mode);
CAFFE2_API Tensor _embedding_bag_dense_backward_cuda(const Tensor & grad, const Tensor & indices, const Tensor & offsets, const Tensor & offset2bag, const Tensor & bag_size, const Tensor & maximum_indices, int64_t num_weights, bool scale_grad_by_freq, int64_t mode);
CAFFE2_API Tensor empty_cpu(IntList size, const TensorOptions & options={});
CAFFE2_API Tensor empty_cuda(IntList size, const TensorOptions & options={});
CAFFE2_API Tensor empty_sparse(IntList size, const TensorOptions & options={});
CAFFE2_API Tensor & resize_cpu_(Tensor & self, IntList size);
CAFFE2_API Tensor & resize_cuda_(Tensor & self, IntList size);
CAFFE2_API Tensor & empty_out(Tensor & result, IntList size);
CAFFE2_API Tensor empty_like(const Tensor & self);
CAFFE2_API Tensor empty_like(const Tensor & self, const TensorOptions & options);
CAFFE2_API Tensor empty_strided(IntList size, IntList stride, const TensorOptions & options={});
CAFFE2_API Tensor erf(const Tensor & self);
CAFFE2_API Tensor & _erf__cpu(Tensor & self);
CAFFE2_API Tensor & _erf__cuda(Tensor & self);
CAFFE2_API Tensor & _erf_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _erf_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor erfc(const Tensor & self);
CAFFE2_API Tensor & _erfc__cpu(Tensor & self);
CAFFE2_API Tensor & _erfc__cuda(Tensor & self);
CAFFE2_API Tensor & _erfc_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _erfc_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor exp(const Tensor & self);
CAFFE2_API Tensor & _exp__cpu(Tensor & self);
CAFFE2_API Tensor & _exp__cuda(Tensor & self);
CAFFE2_API Tensor & _exp_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _exp_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor expm1(const Tensor & self);
CAFFE2_API Tensor & _expm1__cpu(Tensor & self);
CAFFE2_API Tensor & _expm1__cuda(Tensor & self);
CAFFE2_API Tensor & _expm1_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _expm1_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor expand(const Tensor & self, IntList size, bool implicit=false);
CAFFE2_API Tensor expand_as(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor eye(int64_t n, const TensorOptions & options={});
CAFFE2_API Tensor eye(int64_t n, int64_t m, const TensorOptions & options={});
CAFFE2_API Tensor & eye_out_cpu(Tensor & result, int64_t n);
CAFFE2_API Tensor & eye_out_cuda(Tensor & result, int64_t n);
CAFFE2_API Tensor & eye_out_cpu(Tensor & result, int64_t n, int64_t m);
CAFFE2_API Tensor & eye_out_cuda(Tensor & result, int64_t n, int64_t m);
CAFFE2_API Tensor flatten(const Tensor & self, int64_t start_dim=0, int64_t end_dim=-1);
CAFFE2_API Tensor & fill_(Tensor & self, Scalar value);
CAFFE2_API Tensor & fill_(Tensor & self, const Tensor & value);
CAFFE2_API Tensor floor(const Tensor & self);
CAFFE2_API Tensor & _floor__cpu(Tensor & self);
CAFFE2_API Tensor & _floor__cuda(Tensor & self);
CAFFE2_API Tensor & _floor_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _floor_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor full(IntList size, Scalar fill_value, const TensorOptions & options={});
CAFFE2_API Tensor & full_out(Tensor & result, IntList size, Scalar fill_value);
CAFFE2_API Tensor full_like(const Tensor & self, Scalar fill_value);
CAFFE2_API Tensor full_like(const Tensor & self, Scalar fill_value, const TensorOptions & options);
CAFFE2_API Tensor grid_sampler(const Tensor & input, const Tensor & grid, int64_t interpolation_mode, int64_t padding_mode);
CAFFE2_API Tensor grid_sampler_2d_cpu(const Tensor & input, const Tensor & grid, int64_t interpolation_mode, int64_t padding_mode);
CAFFE2_API Tensor grid_sampler_2d_cuda(const Tensor & input, const Tensor & grid, int64_t interpolation_mode, int64_t padding_mode);
CAFFE2_API std::tuple<Tensor,Tensor> grid_sampler_2d_backward_cpu(const Tensor & grad_output, const Tensor & input, const Tensor & grid, int64_t interpolation_mode, int64_t padding_mode);
CAFFE2_API std::tuple<Tensor,Tensor> grid_sampler_2d_backward_cuda(const Tensor & grad_output, const Tensor & input, const Tensor & grid, int64_t interpolation_mode, int64_t padding_mode);
CAFFE2_API Tensor grid_sampler_3d_cpu(const Tensor & input, const Tensor & grid, int64_t interpolation_mode, int64_t padding_mode);
CAFFE2_API Tensor grid_sampler_3d_cuda(const Tensor & input, const Tensor & grid, int64_t interpolation_mode, int64_t padding_mode);
CAFFE2_API std::tuple<Tensor,Tensor> grid_sampler_3d_backward_cpu(const Tensor & grad_output, const Tensor & input, const Tensor & grid, int64_t interpolation_mode, int64_t padding_mode);
CAFFE2_API std::tuple<Tensor,Tensor> grid_sampler_3d_backward_cuda(const Tensor & grad_output, const Tensor & input, const Tensor & grid, int64_t interpolation_mode, int64_t padding_mode);
CAFFE2_API Tensor hann_window(int64_t window_length, const TensorOptions & options={});
CAFFE2_API Tensor hann_window(int64_t window_length, bool periodic, const TensorOptions & options={});
CAFFE2_API Tensor hamming_window(int64_t window_length, const TensorOptions & options={});
CAFFE2_API Tensor hamming_window(int64_t window_length, bool periodic, const TensorOptions & options={});
CAFFE2_API Tensor hamming_window(int64_t window_length, bool periodic, double alpha, const TensorOptions & options={});
CAFFE2_API Tensor hamming_window(int64_t window_length, bool periodic, double alpha, double beta, const TensorOptions & options={});
CAFFE2_API Tensor hinge_embedding_loss(const Tensor & self, const Tensor & target, double margin=1.0, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor ger(const Tensor & self, const Tensor & vec2);
CAFFE2_API Tensor & ger_out(Tensor & result, const Tensor & self, const Tensor & vec2);
CAFFE2_API std::tuple<Tensor,Tensor> gesv(const Tensor & self, const Tensor & A);
CAFFE2_API std::tuple<Tensor &,Tensor &> gesv_out(Tensor & solution, Tensor & lu, const Tensor & self, const Tensor & A);
CAFFE2_API std::tuple<Tensor,Tensor> _gesv_helper_cpu(const Tensor & self, const Tensor & A);
CAFFE2_API std::tuple<Tensor,Tensor> _gesv_helper_cuda(const Tensor & self, const Tensor & A);
CAFFE2_API Tensor group_norm(const Tensor & input, int64_t num_groups, const Tensor & weight={}, const Tensor & bias={}, double eps=1e-05, bool cudnn_enabled=true);
CAFFE2_API Tensor fft(const Tensor & self, int64_t signal_ndim, bool normalized=false);
CAFFE2_API Tensor ifft(const Tensor & self, int64_t signal_ndim, bool normalized=false);
CAFFE2_API Tensor rfft(const Tensor & self, int64_t signal_ndim, bool normalized=false, bool onesided=true);
CAFFE2_API Tensor irfft(const Tensor & self, int64_t signal_ndim, bool normalized=false, bool onesided=true, IntList signal_sizes={});
CAFFE2_API Tensor _fft_mkl(const Tensor & self, int64_t signal_ndim, bool complex_input, bool complex_output, bool inverse, IntList checked_signal_sizes, bool normalized, bool onesided, IntList output_sizes);
CAFFE2_API Tensor _fft_cufft(const Tensor & self, int64_t signal_ndim, bool complex_input, bool complex_output, bool inverse, IntList checked_signal_sizes, bool normalized, bool onesided, IntList output_sizes);
CAFFE2_API int64_t _cufft_get_plan_cache_size();
CAFFE2_API int64_t _cufft_get_plan_cache_max_size();
CAFFE2_API void _cufft_set_plan_cache_max_size(int64_t max_size);
CAFFE2_API void _cufft_clear_plan_cache();
CAFFE2_API Tensor index(const Tensor & self, TensorList indices);
CAFFE2_API Tensor & index_copy_(Tensor & self, int64_t dim, const Tensor & index, const Tensor & source);
CAFFE2_API Tensor index_put(const Tensor & self, TensorList indices, const Tensor & values, bool accumulate=false);
CAFFE2_API Tensor & index_put_(Tensor & self, TensorList indices, const Tensor & values, bool accumulate=false);
CAFFE2_API Tensor instance_norm(const Tensor & input, const Tensor & weight, const Tensor & bias, const Tensor & running_mean, const Tensor & running_var, bool use_input_stats, double momentum, double eps, bool cudnn_enabled);
CAFFE2_API Tensor inverse(const Tensor & self);
CAFFE2_API Tensor & inverse_out(Tensor & result, const Tensor & self);
CAFFE2_API Tensor _inverse_helper_cpu(const Tensor & self);
CAFFE2_API Tensor _inverse_helper_cuda(const Tensor & self);
CAFFE2_API Tensor isclose(const Tensor & self, const Tensor & other, double rtol=1e-05, double atol=1e-08, bool equal_nan=false);
CAFFE2_API bool is_distributed(const Tensor & self);
CAFFE2_API bool is_floating_point(const Tensor & self);
CAFFE2_API bool is_complex(const Tensor & self);
CAFFE2_API bool is_nonzero(const Tensor & self);
CAFFE2_API bool is_same_size(const Tensor & self, const Tensor & other);
CAFFE2_API bool is_signed(const Tensor & self);
CAFFE2_API Tensor kl_div(const Tensor & self, const Tensor & target, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor kl_div_backward_cpu(const Tensor & grad_output, const Tensor & self, const Tensor & target, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor kl_div_backward_cuda(const Tensor & grad_output, const Tensor & self, const Tensor & target, int64_t reduction=Reduction::Mean);
CAFFE2_API std::tuple<Tensor,Tensor> kthvalue(const Tensor & self, int64_t k, int64_t dim=-1, bool keepdim=false);
CAFFE2_API std::tuple<Tensor &,Tensor &> kthvalue_out(Tensor & values, Tensor & indices, const Tensor & self, int64_t k, int64_t dim=-1, bool keepdim=false);
CAFFE2_API Tensor layer_norm(const Tensor & input, IntList normalized_shape, const Tensor & weight={}, const Tensor & bias={}, double eps=1e-05, bool cudnn_enable=true);
CAFFE2_API Tensor linear(const Tensor & input, const Tensor & weight, const Tensor & bias={});
CAFFE2_API Tensor linspace(Scalar start, Scalar end, const TensorOptions & options={});
CAFFE2_API Tensor linspace(Scalar start, Scalar end, int64_t steps, const TensorOptions & options={});
CAFFE2_API Tensor & linspace_out(Tensor & result, Scalar start, Scalar end);
CAFFE2_API Tensor & linspace_out(Tensor & result, Scalar start, Scalar end, int64_t steps);
CAFFE2_API Tensor log(const Tensor & self);
CAFFE2_API Tensor & _log__cpu(Tensor & self);
CAFFE2_API Tensor & _log__cuda(Tensor & self);
CAFFE2_API Tensor & _log_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _log_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor log10(const Tensor & self);
CAFFE2_API Tensor & _log10__cpu(Tensor & self);
CAFFE2_API Tensor & _log10__cuda(Tensor & self);
CAFFE2_API Tensor & _log10_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _log10_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor log1p(const Tensor & self);
CAFFE2_API Tensor & _log1p__cpu(Tensor & self);
CAFFE2_API Tensor & _log1p__cuda(Tensor & self);
CAFFE2_API Tensor & log1p_sparse_(Tensor & self);
CAFFE2_API Tensor & _log1p_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _log1p_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & log1p_out_sparse(Tensor & result, const Tensor & self);
CAFFE2_API Tensor log2(const Tensor & self);
CAFFE2_API Tensor & _log2__cpu(Tensor & self);
CAFFE2_API Tensor & _log2__cuda(Tensor & self);
CAFFE2_API Tensor & _log2_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _log2_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor logdet(const Tensor & self);
CAFFE2_API Tensor logspace(Scalar start, Scalar end, const TensorOptions & options={});
CAFFE2_API Tensor logspace(Scalar start, Scalar end, int64_t steps, const TensorOptions & options={});
CAFFE2_API Tensor & logspace_out(Tensor & result, Scalar start, Scalar end);
CAFFE2_API Tensor & logspace_out(Tensor & result, Scalar start, Scalar end, int64_t steps);
CAFFE2_API Tensor log_softmax(const Tensor & self, int64_t dim, ScalarType dtype);
CAFFE2_API Tensor log_softmax(const Tensor & self, int64_t dim);
CAFFE2_API Tensor log_softmax_cpu(const Tensor & self, int64_t dim, bool half_to_float);
CAFFE2_API Tensor log_softmax_cuda(const Tensor & self, int64_t dim, bool half_to_float);
CAFFE2_API Tensor log_softmax_backward_cpu(const Tensor & grad_output, const Tensor & output, int64_t dim, const Tensor & self);
CAFFE2_API Tensor log_softmax_backward_cuda(const Tensor & grad_output, const Tensor & output, int64_t dim, const Tensor & self);
CAFFE2_API Tensor logsumexp(const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor & logsumexp_out(Tensor & result, const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor margin_ranking_loss(const Tensor & input1, const Tensor & input2, const Tensor & target, double margin=0.0, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor matmul(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & matmul_out(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor matrix_rank(const Tensor & self, double tol, bool symmetric=false);
CAFFE2_API Tensor matrix_rank(const Tensor & self, bool symmetric=false);
CAFFE2_API Tensor matrix_power(const Tensor & self, int64_t n);
CAFFE2_API std::tuple<Tensor,Tensor> max(const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API std::tuple<Tensor &,Tensor &> max_out(Tensor & max, Tensor & max_values, const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor max_values(const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API std::tuple<Tensor,Tensor> max_pool1d_with_indices(const Tensor & self, IntList kernel_size, IntList stride={}, IntList padding=0, IntList dilation=1, bool ceil_mode=false);
CAFFE2_API Tensor max_pool1d(const Tensor & self, IntList kernel_size, IntList stride={}, IntList padding=0, IntList dilation=1, bool ceil_mode=false);
CAFFE2_API Tensor max_pool2d(const Tensor & self, IntList kernel_size, IntList stride={}, IntList padding=0, IntList dilation=1, bool ceil_mode=false);
CAFFE2_API Tensor max_pool3d(const Tensor & self, IntList kernel_size, IntList stride={}, IntList padding=0, IntList dilation=1, bool ceil_mode=false);
CAFFE2_API Tensor mean(const Tensor & self, ScalarType dtype);
CAFFE2_API Tensor mean(const Tensor & self);
CAFFE2_API Tensor mean(const Tensor & self, IntList dim, bool keepdim, ScalarType dtype);
CAFFE2_API Tensor mean(const Tensor & self, IntList dim, bool keepdim=false);
CAFFE2_API Tensor mean(const Tensor & self, IntList dim, ScalarType dtype);
CAFFE2_API Tensor & mean_out(Tensor & result, const Tensor & self, IntList dim, bool keepdim, ScalarType dtype);
CAFFE2_API Tensor & mean_out(Tensor & result, const Tensor & self, IntList dim, bool keepdim=false);
CAFFE2_API Tensor & mean_out(Tensor & result, const Tensor & self, IntList dim, ScalarType dtype);
CAFFE2_API std::tuple<Tensor,Tensor> median(const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API std::tuple<Tensor &,Tensor &> median_out(Tensor & values, Tensor & indices, const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API std::tuple<Tensor,Tensor> min(const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API std::tuple<Tensor &,Tensor &> min_out(Tensor & min, Tensor & min_indices, const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor min_values(const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor mkldnn_convolution(const Tensor & self, const Tensor & weight, const Tensor & bias, IntList padding, IntList stride, IntList dilation, int64_t groups);
CAFFE2_API Tensor mkldnn_convolution_backward_input(IntList self_size, const Tensor & grad_output, const Tensor & weight, IntList padding, IntList stride, IntList dilation, int64_t groups, bool bias_defined);
CAFFE2_API std::tuple<Tensor,Tensor> mkldnn_convolution_backward_weights(IntList weight_size, const Tensor & grad_output, const Tensor & self, IntList padding, IntList stride, IntList dilation, int64_t groups, bool bias_defined);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> mkldnn_convolution_backward(const Tensor & self, const Tensor & grad_output, const Tensor & weight, IntList padding, IntList stride, IntList dilation, int64_t groups, std::array<bool,3> output_mask);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> miopen_batch_norm(const Tensor & input, const Tensor & weight, const Tensor & bias, const Tensor & running_mean, const Tensor & running_var, bool training, double exponential_average_factor, double epsilon);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> miopen_batch_norm_backward(const Tensor & input, const Tensor & grad_output, const Tensor & weight, const Tensor & running_mean, const Tensor & running_var, const Tensor & save_mean, const Tensor & save_var, double epsilon);
CAFFE2_API Tensor miopen_convolution(const Tensor & self, const Tensor & weight, const Tensor & bias, IntList padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic);
CAFFE2_API Tensor miopen_convolution_backward_input(IntList self_size, const Tensor & grad_output, const Tensor & weight, IntList padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> miopen_convolution_backward(const Tensor & self, const Tensor & grad_output, const Tensor & weight, IntList padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic, std::array<bool,3> output_mask);
CAFFE2_API Tensor miopen_convolution_backward_bias(const Tensor & grad_output);
CAFFE2_API Tensor miopen_convolution_backward_weight(IntList weight_size, const Tensor & grad_output, const Tensor & self, IntList padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic);
CAFFE2_API Tensor miopen_convolution_transpose(const Tensor & self, const Tensor & weight, const Tensor & bias, IntList padding, IntList output_padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> miopen_convolution_transpose_backward(const Tensor & self, const Tensor & grad_output, const Tensor & weight, IntList padding, IntList output_padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic, std::array<bool,3> output_mask);
CAFFE2_API Tensor miopen_convolution_transpose_backward_input(const Tensor & grad_output, const Tensor & weight, IntList padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic);
CAFFE2_API Tensor miopen_convolution_transpose_backward_weight(IntList weight_size, const Tensor & grad_output, const Tensor & self, IntList padding, IntList stride, IntList dilation, int64_t groups, bool benchmark, bool deterministic);
CAFFE2_API Tensor mm(const Tensor & self, const Tensor & mat2);
CAFFE2_API Tensor & mm_out(Tensor & result, const Tensor & self, const Tensor & mat2);
CAFFE2_API Tensor _sparse_mm(const Tensor & sparse, const Tensor & dense);
CAFFE2_API std::tuple<Tensor,Tensor> mode(const Tensor & self, int64_t dim=-1, bool keepdim=false);
CAFFE2_API std::tuple<Tensor &,Tensor &> mode_out(Tensor & values, Tensor & indices, const Tensor & self, int64_t dim=-1, bool keepdim=false);
CAFFE2_API Tensor mul(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & mul_(Tensor & self, const Tensor & other);
CAFFE2_API Tensor & mul_out(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor mul(const Tensor & self, Scalar other);
CAFFE2_API Tensor & mul_(Tensor & self, Scalar other);
CAFFE2_API Tensor mv(const Tensor & self, const Tensor & vec);
CAFFE2_API Tensor & mv_out(Tensor & result, const Tensor & self, const Tensor & vec);
CAFFE2_API Tensor mvlgamma(const Tensor & self, int64_t p);
CAFFE2_API Tensor & mvlgamma_(Tensor & self, int64_t p);
CAFFE2_API Tensor narrow_copy_dense(const Tensor & self, int64_t dim, int64_t start, int64_t length);
CAFFE2_API Tensor narrow_copy_sparse(const Tensor & self, int64_t dim, int64_t start, int64_t length);
CAFFE2_API Tensor narrow(const Tensor & self, int64_t dim, int64_t start, int64_t length);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> batch_norm_cpu(const Tensor & input, const Tensor & weight, const Tensor & bias, const Tensor & running_mean, const Tensor & running_var, bool training, double momentum, double eps);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> batch_norm_cuda(const Tensor & input, const Tensor & weight, const Tensor & bias, const Tensor & running_mean, const Tensor & running_var, bool training, double momentum, double eps);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> batch_norm_backward_cpu(const Tensor & grad_out, const Tensor & input, const Tensor & weight, const Tensor & running_mean, const Tensor & running_var, const Tensor & save_mean, const Tensor & save_invstd, bool train, double eps, std::array<bool,3> output_mask);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> batch_norm_backward_cuda(const Tensor & grad_out, const Tensor & input, const Tensor & weight, const Tensor & running_mean, const Tensor & running_var, const Tensor & save_mean, const Tensor & save_invstd, bool train, double eps, std::array<bool,3> output_mask);
CAFFE2_API Tensor ones(IntList size, const TensorOptions & options={});
CAFFE2_API Tensor & ones_out(Tensor & result, IntList size);
CAFFE2_API Tensor ones_like(const Tensor & self);
CAFFE2_API Tensor ones_like(const Tensor & self, const TensorOptions & options);
CAFFE2_API Tensor pairwise_distance(const Tensor & x1, const Tensor & x2, double p=2, double eps=1e-06, bool keepdim=false);
CAFFE2_API Tensor pdist(const Tensor & self, double p=2);
CAFFE2_API Tensor _pdist_forward(const Tensor & self, double p=2);
CAFFE2_API Tensor _pdist_backward(const Tensor & grad, const Tensor & self, double p, const Tensor & pdist);
CAFFE2_API Tensor cosine_similarity(const Tensor & x1, const Tensor & x2, int64_t dim=1, double eps=1e-08);
CAFFE2_API Tensor permute(const Tensor & self, IntList dims);
CAFFE2_API Tensor pixel_shuffle(const Tensor & self, int64_t upscale_factor);
CAFFE2_API Tensor pin_memory(const Tensor & self);
CAFFE2_API Tensor pinverse(const Tensor & self, double rcond=1e-15);
CAFFE2_API Tensor rand(IntList size, const TensorOptions & options={});
CAFFE2_API Tensor rand(IntList size, Generator * generator, const TensorOptions & options={});
CAFFE2_API Tensor & rand_out(Tensor & result, IntList size);
CAFFE2_API Tensor & rand_out(Tensor & result, IntList size, Generator * generator);
CAFFE2_API Tensor rand_like(const Tensor & self);
CAFFE2_API Tensor rand_like(const Tensor & self, const TensorOptions & options);
CAFFE2_API Tensor randint(int64_t high, IntList size, const TensorOptions & options={});
CAFFE2_API Tensor randint(int64_t high, IntList size, Generator * generator, const TensorOptions & options={});
CAFFE2_API Tensor randint(int64_t low, int64_t high, IntList size, const TensorOptions & options={});
CAFFE2_API Tensor randint(int64_t low, int64_t high, IntList size, Generator * generator, const TensorOptions & options={});
CAFFE2_API Tensor & randint_out(Tensor & result, int64_t high, IntList size);
CAFFE2_API Tensor & randint_out(Tensor & result, int64_t high, IntList size, Generator * generator);
CAFFE2_API Tensor & randint_out(Tensor & result, int64_t low, int64_t high, IntList size);
CAFFE2_API Tensor & randint_out(Tensor & result, int64_t low, int64_t high, IntList size, Generator * generator);
CAFFE2_API Tensor randint_like(const Tensor & self, int64_t high);
CAFFE2_API Tensor randint_like(const Tensor & self, int64_t low, int64_t high);
CAFFE2_API Tensor randint_like(const Tensor & self, int64_t high, const TensorOptions & options);
CAFFE2_API Tensor randint_like(const Tensor & self, int64_t low, int64_t high, const TensorOptions & options);
CAFFE2_API Tensor randn(IntList size, const TensorOptions & options={});
CAFFE2_API Tensor randn(IntList size, Generator * generator, const TensorOptions & options={});
CAFFE2_API Tensor & randn_out(Tensor & result, IntList size);
CAFFE2_API Tensor & randn_out(Tensor & result, IntList size, Generator * generator);
CAFFE2_API Tensor randn_like(const Tensor & self);
CAFFE2_API Tensor randn_like(const Tensor & self, const TensorOptions & options);
CAFFE2_API Tensor randperm(int64_t n, const TensorOptions & options={});
CAFFE2_API Tensor randperm(int64_t n, Generator * generator, const TensorOptions & options={});
CAFFE2_API Tensor & randperm_out(Tensor & result, int64_t n);
CAFFE2_API Tensor & randperm_out_cpu(Tensor & result, int64_t n, Generator * generator);
CAFFE2_API Tensor & randperm_out_cuda(Tensor & result, int64_t n, Generator * generator);
CAFFE2_API Tensor range(Scalar start, Scalar end, const TensorOptions & options={});
CAFFE2_API Tensor range(Scalar start, Scalar end, Scalar step, const TensorOptions & options={});
CAFFE2_API Tensor & range_out(Tensor & result, Scalar start, Scalar end);
CAFFE2_API Tensor & range_out(Tensor & result, Scalar start, Scalar end, Scalar step);
CAFFE2_API Tensor repeat(const Tensor & self, IntList repeats);
CAFFE2_API Tensor reshape(const Tensor & self, IntList shape);
CAFFE2_API Tensor reshape_as(const Tensor & self, const Tensor & other);
CAFFE2_API std::tuple<Tensor,Tensor> RoiPooling2d_forward_cpu(const Tensor & input, const Tensor & rois, int64_t pooledHeight, int64_t pooledWidth, double spatialScale);
CAFFE2_API std::tuple<Tensor,Tensor> RoiPooling2d_forward_cuda(const Tensor & input, const Tensor & rois, int64_t pooledHeight, int64_t pooledWidth, double spatialScale);
CAFFE2_API Tensor RoiPooling2d_backward_cpu(const Tensor & input, const Tensor & rois, int64_t pooledHeight, int64_t pooledWidth, double spatialScale, const Tensor & gradOutput, const Tensor & argmaxes);
CAFFE2_API Tensor RoiPooling2d_backward_cuda(const Tensor & input, const Tensor & rois, int64_t pooledHeight, int64_t pooledWidth, double spatialScale, const Tensor & gradOutput, const Tensor & argmaxes);
CAFFE2_API Tensor round(const Tensor & self);
CAFFE2_API Tensor & _round__cpu(Tensor & self);
CAFFE2_API Tensor & _round__cuda(Tensor & self);
CAFFE2_API Tensor & _round_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _round_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor rrelu(const Tensor & self, Scalar lower=0.125, Scalar upper=0.3333333333333333, bool training=false, Generator * generator=nullptr);
CAFFE2_API Tensor & rrelu_(Tensor & self, Scalar lower=0.125, Scalar upper=0.3333333333333333, bool training=false, Generator * generator=nullptr);
CAFFE2_API Tensor relu(const Tensor & self);
CAFFE2_API Tensor & relu_(Tensor & self);
CAFFE2_API Tensor prelu_cpu(const Tensor & self, const Tensor & weight);
CAFFE2_API Tensor prelu_cuda(const Tensor & self, const Tensor & weight);
CAFFE2_API std::tuple<Tensor,Tensor> prelu_backward_cpu(const Tensor & grad_output, const Tensor & self, const Tensor & weight);
CAFFE2_API std::tuple<Tensor,Tensor> prelu_backward_cuda(const Tensor & grad_output, const Tensor & self, const Tensor & weight);
CAFFE2_API Tensor hardshrink_cpu(const Tensor & self, Scalar lambd=0.5);
CAFFE2_API Tensor hardshrink_cuda(const Tensor & self, Scalar lambd=0.5);
CAFFE2_API Tensor hardshrink_backward_cpu(const Tensor & grad_out, const Tensor & self, Scalar lambd);
CAFFE2_API Tensor hardshrink_backward_cuda(const Tensor & grad_out, const Tensor & self, Scalar lambd);
CAFFE2_API Tensor rsqrt(const Tensor & self);
CAFFE2_API Tensor & _rsqrt__cpu(Tensor & self);
CAFFE2_API Tensor & _rsqrt__cuda(Tensor & self);
CAFFE2_API Tensor & _rsqrt_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _rsqrt_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor select(const Tensor & self, int64_t dim, int64_t index);
CAFFE2_API Tensor selu(const Tensor & self);
CAFFE2_API Tensor & selu_(Tensor & self);
CAFFE2_API Tensor celu(const Tensor & self, Scalar alpha=1.0);
CAFFE2_API Tensor & celu_(Tensor & self, Scalar alpha=1.0);
CAFFE2_API Tensor sigmoid(const Tensor & self);
CAFFE2_API Tensor & _sigmoid__cpu(Tensor & self);
CAFFE2_API Tensor & _sigmoid__cuda(Tensor & self);
CAFFE2_API Tensor & _sigmoid_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _sigmoid_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor sin(const Tensor & self);
CAFFE2_API Tensor & _sin__cpu(Tensor & self);
CAFFE2_API Tensor & _sin__cuda(Tensor & self);
CAFFE2_API Tensor & _sin_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _sin_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor sinh(const Tensor & self);
CAFFE2_API Tensor & _sinh__cpu(Tensor & self);
CAFFE2_API Tensor & _sinh__cuda(Tensor & self);
CAFFE2_API Tensor & _sinh_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _sinh_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor detach(const Tensor & self);
CAFFE2_API Tensor & detach_(Tensor & self);
CAFFE2_API int64_t size(const Tensor & self, int64_t dim);
CAFFE2_API Tensor slice(const Tensor & self, int64_t dim=0, int64_t start=0, int64_t end=9223372036854775807, int64_t step=1);
CAFFE2_API std::tuple<Tensor,Tensor> slogdet(const Tensor & self);
CAFFE2_API Tensor smm(const Tensor & self, const Tensor & mat2);
CAFFE2_API Tensor softmax(const Tensor & self, int64_t dim, ScalarType dtype);
CAFFE2_API Tensor softmax(const Tensor & self, int64_t dim);
CAFFE2_API Tensor softmax_cpu(const Tensor & self, int64_t dim, bool half_to_float);
CAFFE2_API Tensor softmax_cuda(const Tensor & self, int64_t dim, bool half_to_float);
CAFFE2_API Tensor softmax_backward_cpu(const Tensor & grad_output, const Tensor & output, int64_t dim, const Tensor & self);
CAFFE2_API Tensor softmax_backward_cuda(const Tensor & grad_output, const Tensor & output, int64_t dim, const Tensor & self);
CAFFE2_API Tensor & add_out_sparse_cpu(Tensor & result, const Tensor & self, const Tensor & other, Scalar alpha=1);
CAFFE2_API Tensor & add_out_sparse_cuda(Tensor & result, const Tensor & self, const Tensor & other, Scalar alpha=1);
CAFFE2_API Tensor & add_out_dense_sparse_cpu(Tensor & result, const Tensor & self, SparseTensorRef other, Scalar alpha=1);
CAFFE2_API Tensor & add_out_dense_sparse_cuda(Tensor & result, const Tensor & self, SparseTensorRef other, Scalar alpha=1);
CAFFE2_API Tensor & div_out_sparse_zerodim(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & div_out_sparse_scalar(Tensor & result, const Tensor & self, Scalar other);
CAFFE2_API Tensor & mul_out_sparse_cpu(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & mul_out_sparse_cuda(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & mul_out_sparse_zerodim(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & mul_out_sparse_scalar(Tensor & result, const Tensor & self, Scalar other);
CAFFE2_API std::vector<Tensor> split(const Tensor & self, int64_t split_size, int64_t dim=0);
CAFFE2_API std::vector<Tensor> split_with_sizes(const Tensor & self, IntList split_sizes, int64_t dim=0);
CAFFE2_API Tensor squeeze(const Tensor & self);
CAFFE2_API Tensor squeeze(const Tensor & self, int64_t dim);
CAFFE2_API Tensor & squeeze_(Tensor & self);
CAFFE2_API Tensor & squeeze_(Tensor & self, int64_t dim);
CAFFE2_API Tensor sspaddmm(const Tensor & self, const Tensor & mat1, const Tensor & mat2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & _sspaddmm_out_only_sparse(Tensor & result, const Tensor & self, const Tensor & mat1, const Tensor & mat2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & _sspaddmm_out_only_sparse_cuda(Tensor & result, const Tensor & self, const Tensor & mat1, const Tensor & mat2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & _sspaddmm_out_cpu(Tensor & result, const Tensor & self, const Tensor & mat1, const Tensor & mat2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & _sspaddmm_out_cuda(Tensor & result, const Tensor & self, const Tensor & mat1, const Tensor & mat2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor stack(TensorList tensors, int64_t dim=0);
CAFFE2_API Tensor & stack_out(Tensor & result, TensorList tensors, int64_t dim=0);
CAFFE2_API Tensor stft(const Tensor & self, int64_t n_fft, int64_t hop_length, int64_t win_length, const Tensor & window={}, bool normalized=false, bool onesided=true);
CAFFE2_API int64_t stride(const Tensor & self, int64_t dim);
CAFFE2_API Tensor sum(const Tensor & self, ScalarType dtype);
CAFFE2_API Tensor sum(const Tensor & self);
CAFFE2_API Tensor sum(const Tensor & self, IntList dim, bool keepdim, ScalarType dtype);
CAFFE2_API Tensor sum(const Tensor & self, IntList dim, bool keepdim=false);
CAFFE2_API Tensor sum(const Tensor & self, IntList dim, ScalarType dtype);
CAFFE2_API Tensor & sum_out(Tensor & result, const Tensor & self, IntList dim, bool keepdim, ScalarType dtype);
CAFFE2_API Tensor & sum_out(Tensor & result, const Tensor & self, IntList dim, bool keepdim=false);
CAFFE2_API Tensor & sum_out(Tensor & result, const Tensor & self, IntList dim, ScalarType dtype);
CAFFE2_API Tensor sqrt(const Tensor & self);
CAFFE2_API Tensor & _sqrt__cpu(Tensor & self);
CAFFE2_API Tensor & _sqrt__cuda(Tensor & self);
CAFFE2_API Tensor & _sqrt_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _sqrt_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor std(const Tensor & self, bool unbiased=true);
CAFFE2_API Tensor std(const Tensor & self, int64_t dim, bool unbiased=true, bool keepdim=false);
CAFFE2_API Tensor & std_out(Tensor & result, const Tensor & self, int64_t dim, bool unbiased=true, bool keepdim=false);
CAFFE2_API Tensor prod(const Tensor & self, ScalarType dtype);
CAFFE2_API Tensor prod(const Tensor & self);
CAFFE2_API Tensor prod(const Tensor & self, int64_t dim, bool keepdim, ScalarType dtype);
CAFFE2_API Tensor prod(const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor prod(const Tensor & self, int64_t dim, ScalarType dtype);
CAFFE2_API Tensor & prod_out(Tensor & result, const Tensor & self, int64_t dim, bool keepdim, ScalarType dtype);
CAFFE2_API Tensor & prod_out(Tensor & result, const Tensor & self, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor & prod_out(Tensor & result, const Tensor & self, int64_t dim, ScalarType dtype);
CAFFE2_API Tensor t(const Tensor & self);
CAFFE2_API Tensor & t_(Tensor & self);
CAFFE2_API Tensor tan(const Tensor & self);
CAFFE2_API Tensor & _tan__cpu(Tensor & self);
CAFFE2_API Tensor & _tan__cuda(Tensor & self);
CAFFE2_API Tensor & _tan_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _tan_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor tanh(const Tensor & self);
CAFFE2_API Tensor & _tanh__cpu(Tensor & self);
CAFFE2_API Tensor & _tanh__cuda(Tensor & self);
CAFFE2_API Tensor & _tanh_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _tanh_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor tensordot(const Tensor & self, const Tensor & other, IntList dims_self, IntList dims_other);
CAFFE2_API Tensor threshold(const Tensor & self, Scalar threshold, Scalar value);
CAFFE2_API Tensor & threshold_(Tensor & self, Scalar threshold, Scalar value);
CAFFE2_API Tensor & threshold_out(Tensor & result, const Tensor & self, Scalar threshold, Scalar value);
CAFFE2_API Tensor threshold_backward(const Tensor & grad_output, const Tensor & self, Scalar threshold);
CAFFE2_API Tensor transpose(const Tensor & self, int64_t dim0, int64_t dim1);
CAFFE2_API Tensor & transpose_(Tensor & self, int64_t dim0, int64_t dim1);
CAFFE2_API Tensor flip_cpu(const Tensor & self, IntList dims);
CAFFE2_API Tensor flip_cuda(const Tensor & self, IntList dims);
CAFFE2_API Tensor roll_cpu(const Tensor & self, IntList shifts, IntList dims={});
CAFFE2_API Tensor roll_cuda(const Tensor & self, IntList shifts, IntList dims={});
CAFFE2_API Tensor rot90(const Tensor & self, int64_t k=1, IntList dims={0,1});
CAFFE2_API Tensor _trilinear(const Tensor & i1, const Tensor & i2, const Tensor & i3, IntList expand1, IntList expand2, IntList expand3, IntList sumdim, int64_t unroll_dim=1);
CAFFE2_API Tensor triplet_margin_loss(const Tensor & anchor, const Tensor & positive, const Tensor & negative, double margin=1.0, double p=2, double eps=1e-06, bool swap=false, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor trunc(const Tensor & self);
CAFFE2_API Tensor & _trunc__cpu(Tensor & self);
CAFFE2_API Tensor & _trunc__cuda(Tensor & self);
CAFFE2_API Tensor & _trunc_out_cpu(Tensor & result, const Tensor & self);
CAFFE2_API Tensor & _trunc_out_cuda(Tensor & result, const Tensor & self);
CAFFE2_API Tensor type_as(const Tensor & self, const Tensor & other);
CAFFE2_API std::tuple<Tensor,Tensor> _unique_cpu(const Tensor & self, bool sorted=false, bool return_inverse=false);
CAFFE2_API std::tuple<Tensor,Tensor> _unique_cuda(const Tensor & self, bool sorted=false, bool return_inverse=false);
CAFFE2_API std::tuple<Tensor,Tensor> _unique_dim_cpu(const Tensor & self, int64_t dim, bool sorted=false, bool return_inverse=false);
CAFFE2_API std::tuple<Tensor,Tensor> _unique_dim_cuda(const Tensor & self, int64_t dim, bool sorted=false, bool return_inverse=false);
CAFFE2_API Tensor _unsafe_view(const Tensor & self, IntList size);
CAFFE2_API Tensor unsqueeze(const Tensor & self, int64_t dim);
CAFFE2_API Tensor & unsqueeze_(Tensor & self, int64_t dim);
CAFFE2_API Tensor var(const Tensor & self, bool unbiased=true);
CAFFE2_API Tensor var(const Tensor & self, int64_t dim, bool unbiased=true, bool keepdim=false);
CAFFE2_API Tensor & var_out(Tensor & result, const Tensor & self, int64_t dim, bool unbiased=true, bool keepdim=false);
CAFFE2_API Tensor view_as(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor where(const Tensor & condition, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor _s_where_cpu(const Tensor & condition, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor _s_where_cuda(const Tensor & condition, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor norm_except_dim(const Tensor & v, int64_t pow=2, int64_t dim=0);
CAFFE2_API Tensor _weight_norm(const Tensor & v, const Tensor & g, int64_t dim=0);
CAFFE2_API std::tuple<Tensor,Tensor> weight_norm_cuda(const Tensor & v, const Tensor & g, int64_t dim=0);
CAFFE2_API std::tuple<Tensor,Tensor> weight_norm_cuda_backward(const Tensor & grad_w, const Tensor & saved_v, const Tensor & saved_g, const Tensor & saved_norms, int64_t dim);
CAFFE2_API std::tuple<Tensor,Tensor> _weight_norm_differentiable_backward(const Tensor & grad_w, const Tensor & saved_v, const Tensor & saved_g, const Tensor & saved_norms, int64_t dim);
CAFFE2_API Tensor zeros(IntList size, const TensorOptions & options={});
CAFFE2_API Tensor & zeros_out(Tensor & result, IntList size);
CAFFE2_API Tensor zeros_like(const Tensor & self);
CAFFE2_API Tensor zeros_like(const Tensor & self, const TensorOptions & options);
CAFFE2_API Tensor _standard_gamma_grad_cpu(const Tensor & self, const Tensor & output);
CAFFE2_API Tensor _standard_gamma_grad_cuda(const Tensor & self, const Tensor & output);
CAFFE2_API Tensor _s_gamma_cpu(const Tensor & self, Generator * generator=nullptr);
CAFFE2_API Tensor _s_gamma_cuda(const Tensor & self, Generator * generator=nullptr);
CAFFE2_API Tensor _s_poisson_cpu(const Tensor & self, Generator * generator=nullptr);
CAFFE2_API Tensor _s_poisson_cuda(const Tensor & self, Generator * generator=nullptr);
CAFFE2_API Tensor norm_sparse(const Tensor & self, Scalar p=2);
CAFFE2_API Tensor _sparse_sum(const Tensor & self);
CAFFE2_API Tensor _sparse_sum(const Tensor & self, ScalarType dtype);
CAFFE2_API Tensor _sparse_sum(const Tensor & self, IntList dim);
CAFFE2_API Tensor _sparse_sum(const Tensor & self, IntList dim, ScalarType dtype);
CAFFE2_API Tensor _sparse_sum_backward_cpu(const Tensor & grad, const Tensor & self, IntList dim);
CAFFE2_API Tensor _sparse_sum_backward_cuda(const Tensor & grad, const Tensor & self, IntList dim);
CAFFE2_API Tensor norm(const Tensor & self, Scalar p=2);
CAFFE2_API Tensor norm(const Tensor & self, Scalar p, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor & norm_out(Tensor & result, const Tensor & self, Scalar p, int64_t dim, bool keepdim=false);
CAFFE2_API Tensor frobenius_norm(const Tensor & self);
CAFFE2_API Tensor frobenius_norm(const Tensor & self, IntList dim, bool keepdim=false);
CAFFE2_API Tensor & frobenius_norm_out(Tensor & result, const Tensor & self, IntList dim, bool keepdim=false);
CAFFE2_API Tensor nuclear_norm(const Tensor & self, bool keepdim=false);
CAFFE2_API Tensor & nuclear_norm_out(Tensor & result, const Tensor & self, bool keepdim=false);
CAFFE2_API Tensor clone_sparse(const Tensor & self);
CAFFE2_API Tensor clone(const Tensor & self);
CAFFE2_API Tensor & resize_as_sparse_(Tensor & self, const Tensor & the_template);
CAFFE2_API Tensor & resize_as_(Tensor & self, const Tensor & the_template);
CAFFE2_API Tensor & pow_out_sparse_scalar(Tensor & result, const Tensor & self, Scalar exponent);
CAFFE2_API Tensor pow_sparse_scalar(const Tensor & self, Scalar exponent);
CAFFE2_API Tensor & pow_out(Tensor & result, const Tensor & self, Scalar exponent);
CAFFE2_API Tensor pow(const Tensor & self, Scalar exponent);
CAFFE2_API Tensor & zero_sparse_(Tensor & self);
CAFFE2_API Tensor & zero_(Tensor & self);
CAFFE2_API Tensor & sub_out(Tensor & result, const Tensor & self, const Tensor & other, Scalar alpha=1);
CAFFE2_API Tensor sub(const Tensor & self, const Tensor & other, Scalar alpha=1);
CAFFE2_API Tensor & sub_(Tensor & self, const Tensor & other, Scalar alpha=1);
CAFFE2_API Tensor sub(const Tensor & self, Scalar other, Scalar alpha=1);
CAFFE2_API Tensor & sub_(Tensor & self, Scalar other, Scalar alpha=1);
CAFFE2_API Tensor rsub(const Tensor & self, const Tensor & other, Scalar alpha=1);
CAFFE2_API Tensor rsub(const Tensor & self, Scalar other, Scalar alpha=1);
CAFFE2_API Tensor & s_addmm_out_sparse_dense_cpu(Tensor & result, const Tensor & self, const Tensor & mat1, const Tensor & mat2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & s_addmm_out_sparse_dense_cuda(Tensor & result, const Tensor & self, const Tensor & mat1, const Tensor & mat2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor s_addmm_sparse_dense_cpu(const Tensor & self, const Tensor & mat1, const Tensor & mat2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor s_addmm_sparse_dense_cuda(const Tensor & self, const Tensor & mat1, const Tensor & mat2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & s_addmm_sparse_dense_cpu_(Tensor & self, const Tensor & mat1, const Tensor & mat2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & s_addmm_sparse_dense_cuda_(Tensor & self, const Tensor & mat1, const Tensor & mat2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor _sparse_addmm(const Tensor & self, const Tensor & sparse, const Tensor & dense, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & addmm_out(Tensor & result, const Tensor & self, const Tensor & mat1, const Tensor & mat2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor addmm(const Tensor & self, const Tensor & mat1, const Tensor & mat2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & addmm_(Tensor & self, const Tensor & mat1, const Tensor & mat2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor sparse_coo_tensor(IntList size, const TensorOptions & options);
CAFFE2_API Tensor sparse_coo_tensor(const Tensor & indices, const Tensor & values, const TensorOptions & options={});
CAFFE2_API Tensor sparse_coo_tensor(const Tensor & indices, const Tensor & values, IntList size, const TensorOptions & options={});
CAFFE2_API Tensor _sparse_coo_tensor_unsafe(const Tensor & indices, const Tensor & values, IntList size, const TensorOptions & options={});
CAFFE2_API Tensor new_with_dims_sparse(int64_t sparse_dim, int64_t dense_dim, IntList size, const TensorOptions & options);
CAFFE2_API Tensor new_with_dims_and_tensor_sparse(int64_t sparse_dim, int64_t dense_dim, IntList size, const Tensor & indices, const Tensor & values, const TensorOptions & options);
CAFFE2_API Tensor & sparse_resize_(Tensor & self, IntList size, int64_t sparse_dim, int64_t dense_dim);
CAFFE2_API Tensor & sparse_resize_and_clear_(Tensor & self, IntList size, int64_t sparse_dim, int64_t dense_dim);
CAFFE2_API Tensor sparse_mask_cpu(const Tensor & self, SparseTensorRef mask);
CAFFE2_API Tensor sparse_mask_cuda(const Tensor & self, SparseTensorRef mask);
CAFFE2_API Tensor sparse_to_dense(const Tensor & self);
CAFFE2_API int64_t sparse_dim_sparse(const Tensor & self);
CAFFE2_API int64_t sparse_dim_sparse(const Tensor & self);
CAFFE2_API int64_t dense_dim_sparse(const Tensor & self);
CAFFE2_API int64_t dense_dim_sparse(const Tensor & self);
CAFFE2_API int64_t _nnz_sparse(const Tensor & self);
CAFFE2_API Tensor coalesce_sparse_cpu(const Tensor & self);
CAFFE2_API Tensor coalesce_sparse_cuda(const Tensor & self);
CAFFE2_API bool is_coalesced_sparse(const Tensor & self);
CAFFE2_API Tensor _indices_sparse(const Tensor & self);
CAFFE2_API Tensor _values_sparse(const Tensor & self);
CAFFE2_API Tensor & _coalesced_sparse_(Tensor & self, bool coalesced);
CAFFE2_API Tensor indices_sparse(const Tensor & self);
CAFFE2_API Tensor values_sparse(const Tensor & self);
CAFFE2_API Tensor & hspmm_out_sparse_cpu(Tensor & result, const Tensor & mat1, const Tensor & mat2);
CAFFE2_API Tensor & hspmm_out_sparse_cuda(Tensor & result, const Tensor & mat1, const Tensor & mat2);
CAFFE2_API Tensor hspmm_sparse_cpu(const Tensor & mat1, const Tensor & mat2);
CAFFE2_API Tensor hspmm_sparse_cuda(const Tensor & mat1, const Tensor & mat2);
CAFFE2_API Tensor & copy_sparse_(Tensor & self, const Tensor & src, bool non_blocking=false);
CAFFE2_API int64_t numel(const Tensor & self);
CAFFE2_API std::vector<Tensor> unbind(const Tensor & self, int64_t dim=0);
CAFFE2_API Tensor dense_to_sparse(const Tensor & self, int64_t sparse_dim);
CAFFE2_API Tensor dense_to_sparse(const Tensor & self);
CAFFE2_API Tensor to(const Tensor & self, const TensorOptions & options, bool non_blocking=false, bool copy=false);
CAFFE2_API Tensor to(const Tensor & self, Device device, ScalarType dtype, bool non_blocking=false, bool copy=false);
CAFFE2_API Tensor to(const Tensor & self, ScalarType dtype, bool non_blocking=false, bool copy=false);
CAFFE2_API Tensor to(const Tensor & self, const Tensor & other, bool non_blocking=false, bool copy=false);
CAFFE2_API std::vector<Tensor> meshgrid(TensorList tensors);
CAFFE2_API Scalar item(const Tensor & self);
CAFFE2_API Scalar _local_scalar_dense_cpu(const Tensor & self);
CAFFE2_API Scalar _local_scalar_dense_cuda(const Tensor & self);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> _thnn_fused_lstm_cell_cuda(const Tensor & input_gates, const Tensor & hidden_gates, const Tensor & cx, const Tensor & input_bias={}, const Tensor & hidden_bias={});
CAFFE2_API std::tuple<Tensor,Tensor,Tensor,Tensor,Tensor> _thnn_fused_lstm_cell_backward_cuda(const Tensor & grad_hy, const Tensor & grad_cy, const Tensor & cx, const Tensor & cy, const Tensor & workspace, bool has_bias);
CAFFE2_API std::tuple<Tensor,Tensor> _thnn_fused_gru_cell_cuda(const Tensor & input_gates, const Tensor & hidden_gates, const Tensor & hx, const Tensor & input_bias={}, const Tensor & hidden_bias={});
CAFFE2_API std::tuple<Tensor,Tensor,Tensor,Tensor,Tensor> _thnn_fused_gru_cell_backward_cuda(const Tensor & grad_hy, const Tensor & workspace, bool has_bias);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> lstm(const Tensor & input, TensorList hx, TensorList params, bool has_biases, int64_t num_layers, double dropout, bool train, bool bidirectional, bool batch_first);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> lstm(const Tensor & data, const Tensor & batch_sizes, TensorList hx, TensorList params, bool has_biases, int64_t num_layers, double dropout, bool train, bool bidirectional);
CAFFE2_API std::tuple<Tensor,Tensor> gru(const Tensor & input, const Tensor & hx, TensorList params, bool has_biases, int64_t num_layers, double dropout, bool train, bool bidirectional, bool batch_first);
CAFFE2_API std::tuple<Tensor,Tensor> gru(const Tensor & data, const Tensor & batch_sizes, const Tensor & hx, TensorList params, bool has_biases, int64_t num_layers, double dropout, bool train, bool bidirectional);
CAFFE2_API std::tuple<Tensor,Tensor> rnn_tanh(const Tensor & input, const Tensor & hx, TensorList params, bool has_biases, int64_t num_layers, double dropout, bool train, bool bidirectional, bool batch_first);
CAFFE2_API std::tuple<Tensor,Tensor> rnn_tanh(const Tensor & data, const Tensor & batch_sizes, const Tensor & hx, TensorList params, bool has_biases, int64_t num_layers, double dropout, bool train, bool bidirectional);
CAFFE2_API std::tuple<Tensor,Tensor> rnn_relu(const Tensor & input, const Tensor & hx, TensorList params, bool has_biases, int64_t num_layers, double dropout, bool train, bool bidirectional, bool batch_first);
CAFFE2_API std::tuple<Tensor,Tensor> rnn_relu(const Tensor & data, const Tensor & batch_sizes, const Tensor & hx, TensorList params, bool has_biases, int64_t num_layers, double dropout, bool train, bool bidirectional);
CAFFE2_API std::tuple<Tensor,Tensor> lstm_cell(const Tensor & input, TensorList hx, const Tensor & w_ih, const Tensor & w_hh, const Tensor & b_ih={}, const Tensor & b_hh={});
CAFFE2_API Tensor gru_cell(const Tensor & input, const Tensor & hx, const Tensor & w_ih, const Tensor & w_hh, const Tensor & b_ih={}, const Tensor & b_hh={});
CAFFE2_API Tensor rnn_tanh_cell(const Tensor & input, const Tensor & hx, const Tensor & w_ih, const Tensor & w_hh, const Tensor & b_ih={}, const Tensor & b_hh={});
CAFFE2_API Tensor rnn_relu_cell(const Tensor & input, const Tensor & hx, const Tensor & w_ih, const Tensor & w_hh, const Tensor & b_ih={}, const Tensor & b_hh={});
CAFFE2_API std::tuple<Tensor,Tensor> _pack_padded_sequence(const Tensor & input, const Tensor & lengths, bool batch_first);
CAFFE2_API Tensor _pack_padded_sequence_backward(const Tensor & grad, IntList input_size, const Tensor & batch_sizes, bool batch_first);
CAFFE2_API std::tuple<Tensor,Tensor> _pad_packed_sequence(const Tensor & data, const Tensor & batch_sizes, bool batch_first, Scalar padding_value, int64_t total_length);
CAFFE2_API void* data_ptr(const Tensor & self);
CAFFE2_API Tensor & set_(Tensor & self, Storage source);
CAFFE2_API Tensor & set_(Tensor & self, Storage source, int64_t storage_offset, IntList size, IntList stride={});
CAFFE2_API Tensor & set_(Tensor & self, const Tensor & source);
CAFFE2_API Tensor & set_(Tensor & self);
CAFFE2_API bool is_set_to(const Tensor & self, const Tensor & tensor);
CAFFE2_API Tensor & masked_fill_(Tensor & self, const Tensor & mask, Scalar value);
CAFFE2_API Tensor & masked_fill_(Tensor & self, const Tensor & mask, const Tensor & value);
CAFFE2_API Tensor & masked_scatter_(Tensor & self, const Tensor & mask, const Tensor & source);
CAFFE2_API Tensor view(const Tensor & self, IntList size);
CAFFE2_API Tensor & put_(Tensor & self, const Tensor & index, const Tensor & source, bool accumulate=false);
CAFFE2_API Tensor & index_add_(Tensor & self, int64_t dim, const Tensor & index, const Tensor & source);
CAFFE2_API Tensor & index_fill_(Tensor & self, int64_t dim, const Tensor & index, Scalar value);
CAFFE2_API Tensor & index_fill_(Tensor & self, int64_t dim, const Tensor & index, const Tensor & value);
CAFFE2_API Tensor & scatter_(Tensor & self, int64_t dim, const Tensor & index, const Tensor & src);
CAFFE2_API Tensor & scatter_(Tensor & self, int64_t dim, const Tensor & index, Scalar value);
CAFFE2_API Tensor & scatter_add_(Tensor & self, int64_t dim, const Tensor & index, const Tensor & src);
CAFFE2_API Tensor & lt_(Tensor & self, Scalar other);
CAFFE2_API Tensor & lt_(Tensor & self, const Tensor & other);
CAFFE2_API Tensor & gt_(Tensor & self, Scalar other);
CAFFE2_API Tensor & gt_(Tensor & self, const Tensor & other);
CAFFE2_API Tensor & le_(Tensor & self, Scalar other);
CAFFE2_API Tensor & le_(Tensor & self, const Tensor & other);
CAFFE2_API Tensor & ge_(Tensor & self, Scalar other);
CAFFE2_API Tensor & ge_(Tensor & self, const Tensor & other);
CAFFE2_API Tensor & eq_(Tensor & self, Scalar other);
CAFFE2_API Tensor & eq_(Tensor & self, const Tensor & other);
CAFFE2_API Tensor & ne_(Tensor & self, Scalar other);
CAFFE2_API Tensor & ne_(Tensor & self, const Tensor & other);
CAFFE2_API Tensor __and__(const Tensor & self, Scalar other);
CAFFE2_API Tensor __and__(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & __iand__(Tensor & self, Scalar other);
CAFFE2_API Tensor & __iand__(Tensor & self, const Tensor & other);
CAFFE2_API Tensor __or__(const Tensor & self, Scalar other);
CAFFE2_API Tensor __or__(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & __ior__(Tensor & self, Scalar other);
CAFFE2_API Tensor & __ior__(Tensor & self, const Tensor & other);
CAFFE2_API Tensor __xor__(const Tensor & self, Scalar other);
CAFFE2_API Tensor __xor__(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & __ixor__(Tensor & self, Scalar other);
CAFFE2_API Tensor & __ixor__(Tensor & self, const Tensor & other);
CAFFE2_API Tensor __lshift__(const Tensor & self, Scalar other);
CAFFE2_API Tensor __lshift__(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & __ilshift__(Tensor & self, Scalar other);
CAFFE2_API Tensor & __ilshift__(Tensor & self, const Tensor & other);
CAFFE2_API Tensor __rshift__(const Tensor & self, Scalar other);
CAFFE2_API Tensor __rshift__(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & __irshift__(Tensor & self, Scalar other);
CAFFE2_API Tensor & __irshift__(Tensor & self, const Tensor & other);
CAFFE2_API Tensor & lgamma_(Tensor & self);
CAFFE2_API Tensor & atan2_(Tensor & self, const Tensor & other);
CAFFE2_API Tensor & tril_cpu_(Tensor & self, int64_t diagonal=0);
CAFFE2_API Tensor & tril_cuda_(Tensor & self, int64_t diagonal=0);
CAFFE2_API Tensor & triu_cpu_(Tensor & self, int64_t diagonal=0);
CAFFE2_API Tensor & triu_cuda_(Tensor & self, int64_t diagonal=0);
CAFFE2_API Tensor & digamma_(Tensor & self);
CAFFE2_API Tensor & polygamma_(Tensor & self, int64_t n);
CAFFE2_API Tensor & erfinv_(Tensor & self);
CAFFE2_API Tensor & frac_(Tensor & self);
CAFFE2_API Tensor & renorm_(Tensor & self, Scalar p, int64_t dim, Scalar maxnorm);
CAFFE2_API Tensor & reciprocal_(Tensor & self);
CAFFE2_API Tensor & neg_(Tensor & self);
CAFFE2_API Tensor & pow_(Tensor & self, Scalar exponent);
CAFFE2_API Tensor & pow_(Tensor & self, const Tensor & exponent);
CAFFE2_API Tensor & lerp_(Tensor & self, const Tensor & end, Scalar weight);
CAFFE2_API Tensor & sign_(Tensor & self);
CAFFE2_API Tensor & fmod_(Tensor & self, Scalar other);
CAFFE2_API Tensor & fmod_(Tensor & self, const Tensor & other);
CAFFE2_API Tensor & remainder_(Tensor & self, Scalar other);
CAFFE2_API Tensor & remainder_(Tensor & self, const Tensor & other);
CAFFE2_API Tensor & addbmm_(Tensor & self, const Tensor & batch1, const Tensor & batch2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & addbmm_out(Tensor & result, const Tensor & self, const Tensor & batch1, const Tensor & batch2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor addbmm(const Tensor & self, const Tensor & batch1, const Tensor & batch2, Scalar beta=1, Scalar alpha=1);
CAFFE2_API Tensor & addcmul_(Tensor & self, const Tensor & tensor1, const Tensor & tensor2, Scalar value=1);
CAFFE2_API Tensor & addcdiv_(Tensor & self, const Tensor & tensor1, const Tensor & tensor2, Scalar value=1);
CAFFE2_API Tensor & random_(Tensor & self, int64_t from, int64_t to, Generator * generator=nullptr);
CAFFE2_API Tensor & random_(Tensor & self, int64_t to, Generator * generator=nullptr);
CAFFE2_API Tensor & random_(Tensor & self, Generator * generator=nullptr);
CAFFE2_API Tensor & uniform_(Tensor & self, double from=0, double to=1, Generator * generator=nullptr);
CAFFE2_API Tensor & normal_(Tensor & self, double mean=0, double std=1, Generator * generator=nullptr);
CAFFE2_API Tensor & cauchy_(Tensor & self, double median=0, double sigma=1, Generator * generator=nullptr);
CAFFE2_API Tensor & log_normal_(Tensor & self, double mean=1, double std=2, Generator * generator=nullptr);
CAFFE2_API Tensor & exponential_(Tensor & self, double lambd=1, Generator * generator=nullptr);
CAFFE2_API Tensor & geometric_(Tensor & self, double p, Generator * generator=nullptr);
CAFFE2_API Tensor & diag_out(Tensor & result, const Tensor & self, int64_t diagonal=0);
CAFFE2_API Tensor diag(const Tensor & self, int64_t diagonal=0);
CAFFE2_API Tensor & cross_out(Tensor & result, const Tensor & self, const Tensor & other, int64_t dim=-1);
CAFFE2_API Tensor cross(const Tensor & self, const Tensor & other, int64_t dim=-1);
CAFFE2_API Tensor & triu_cpu_out(Tensor & result, const Tensor & self, int64_t diagonal=0);
CAFFE2_API Tensor & triu_cuda_out(Tensor & result, const Tensor & self, int64_t diagonal=0);
CAFFE2_API Tensor triu(const Tensor & self, int64_t diagonal=0);
CAFFE2_API Tensor & tril_cpu_out(Tensor & result, const Tensor & self, int64_t diagonal=0);
CAFFE2_API Tensor & tril_cuda_out(Tensor & result, const Tensor & self, int64_t diagonal=0);
CAFFE2_API Tensor tril(const Tensor & self, int64_t diagonal=0);
CAFFE2_API Tensor trace(const Tensor & self);
CAFFE2_API Tensor & ne_out(Tensor & result, const Tensor & self, Scalar other);
CAFFE2_API Tensor ne(const Tensor & self, Scalar other);
CAFFE2_API Tensor & ne_out(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor ne(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & eq_out(Tensor & result, const Tensor & self, Scalar other);
CAFFE2_API Tensor eq(const Tensor & self, Scalar other);
CAFFE2_API Tensor & eq_out(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor eq(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & ge_out(Tensor & result, const Tensor & self, Scalar other);
CAFFE2_API Tensor ge(const Tensor & self, Scalar other);
CAFFE2_API Tensor & ge_out(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor ge(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & le_out(Tensor & result, const Tensor & self, Scalar other);
CAFFE2_API Tensor le(const Tensor & self, Scalar other);
CAFFE2_API Tensor & le_out(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor le(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & gt_out(Tensor & result, const Tensor & self, Scalar other);
CAFFE2_API Tensor gt(const Tensor & self, Scalar other);
CAFFE2_API Tensor & gt_out(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor gt(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & lt_out(Tensor & result, const Tensor & self, Scalar other);
CAFFE2_API Tensor lt(const Tensor & self, Scalar other);
CAFFE2_API Tensor & lt_out(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor lt(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & take_out(Tensor & result, const Tensor & self, const Tensor & index);
CAFFE2_API Tensor take(const Tensor & self, const Tensor & index);
CAFFE2_API Tensor & index_select_out(Tensor & result, const Tensor & self, int64_t dim, const Tensor & index);
CAFFE2_API Tensor index_select(const Tensor & self, int64_t dim, const Tensor & index);
CAFFE2_API Tensor & masked_select_out(Tensor & result, const Tensor & self, const Tensor & mask);
CAFFE2_API Tensor masked_select(const Tensor & self, const Tensor & mask);
CAFFE2_API Tensor & nonzero_out(Tensor & result, const Tensor & self);
CAFFE2_API Tensor nonzero(const Tensor & self);
CAFFE2_API Tensor & gather_out(Tensor & result, const Tensor & self, int64_t dim, const Tensor & index);
CAFFE2_API Tensor gather(const Tensor & self, int64_t dim, const Tensor & index);
CAFFE2_API Tensor & addcmul_out(Tensor & result, const Tensor & self, const Tensor & tensor1, const Tensor & tensor2, Scalar value=1);
CAFFE2_API Tensor addcmul(const Tensor & self, const Tensor & tensor1, const Tensor & tensor2, Scalar value=1);
CAFFE2_API Tensor & addcdiv_out(Tensor & result, const Tensor & self, const Tensor & tensor1, const Tensor & tensor2, Scalar value=1);
CAFFE2_API Tensor addcdiv(const Tensor & self, const Tensor & tensor1, const Tensor & tensor2, Scalar value=1);
CAFFE2_API std::tuple<Tensor &,Tensor &> gels_out(Tensor & X, Tensor & qr, const Tensor & self, const Tensor & A);
CAFFE2_API std::tuple<Tensor,Tensor> gels(const Tensor & self, const Tensor & A);
CAFFE2_API std::tuple<Tensor &,Tensor &> trtrs_out(Tensor & X, Tensor & M, const Tensor & self, const Tensor & A, bool upper=true, bool transpose=false, bool unitriangular=false);
CAFFE2_API std::tuple<Tensor,Tensor> trtrs(const Tensor & self, const Tensor & A, bool upper=true, bool transpose=false, bool unitriangular=false);
CAFFE2_API std::tuple<Tensor &,Tensor &> symeig_out(Tensor & e, Tensor & V, const Tensor & self, bool eigenvectors=false, bool upper=true);
CAFFE2_API std::tuple<Tensor,Tensor> symeig(const Tensor & self, bool eigenvectors=false, bool upper=true);
CAFFE2_API std::tuple<Tensor &,Tensor &> eig_out(Tensor & e, Tensor & v, const Tensor & self, bool eigenvectors=false);
CAFFE2_API std::tuple<Tensor,Tensor> eig(const Tensor & self, bool eigenvectors=false);
CAFFE2_API std::tuple<Tensor &,Tensor &,Tensor &> svd_out(Tensor & U, Tensor & S, Tensor & V, const Tensor & self, bool some=true, bool compute_uv=true);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> svd(const Tensor & self, bool some=true, bool compute_uv=true);
CAFFE2_API Tensor & cholesky_out(Tensor & result, const Tensor & self, bool upper=false);
CAFFE2_API Tensor cholesky(const Tensor & self, bool upper=false);
CAFFE2_API Tensor _cholesky_helper_cpu(const Tensor & self, bool upper);
CAFFE2_API Tensor _cholesky_helper_cuda(const Tensor & self, bool upper);
CAFFE2_API Tensor & potrs_out(Tensor & result, const Tensor & self, const Tensor & input2, bool upper=true);
CAFFE2_API Tensor potrs(const Tensor & self, const Tensor & input2, bool upper=true);
CAFFE2_API Tensor _potrs_helper_cpu(const Tensor & self, const Tensor & A, bool upper);
CAFFE2_API Tensor _potrs_helper_cuda(const Tensor & self, const Tensor & A, bool upper);
CAFFE2_API Tensor & potri_out(Tensor & result, const Tensor & self, bool upper=true);
CAFFE2_API Tensor potri(const Tensor & self, bool upper=true);
CAFFE2_API std::tuple<Tensor &,Tensor &> pstrf_out(Tensor & u, Tensor & piv, const Tensor & self, bool upper=true, Scalar tol=-1);
CAFFE2_API std::tuple<Tensor,Tensor> pstrf(const Tensor & self, bool upper=true, Scalar tol=-1);
CAFFE2_API std::tuple<Tensor &,Tensor &> qr_out(Tensor & Q, Tensor & R, const Tensor & self);
CAFFE2_API std::tuple<Tensor,Tensor> qr(const Tensor & self);
CAFFE2_API std::tuple<Tensor &,Tensor &> geqrf_out(Tensor & result0, Tensor & result1, const Tensor & self);
CAFFE2_API std::tuple<Tensor,Tensor> geqrf(const Tensor & self);
CAFFE2_API Tensor & orgqr_out(Tensor & result, const Tensor & self, const Tensor & input2);
CAFFE2_API Tensor orgqr(const Tensor & self, const Tensor & input2);
CAFFE2_API Tensor & ormqr_out(Tensor & result, const Tensor & self, const Tensor & input2, const Tensor & input3, bool left=true, bool transpose=false);
CAFFE2_API Tensor ormqr(const Tensor & self, const Tensor & input2, const Tensor & input3, bool left=true, bool transpose=false);
CAFFE2_API std::tuple<Tensor &,Tensor &> btrifact_out(Tensor & A_LU, Tensor & pivots, const Tensor & self, bool pivot=true);
CAFFE2_API std::tuple<Tensor,Tensor> btrifact(const Tensor & self, bool pivot=true);
CAFFE2_API std::tuple<Tensor &,Tensor &,Tensor &> btrifact_with_info_out(Tensor & A_LU, Tensor & pivots, Tensor & info, const Tensor & self, bool pivot=true);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> btrifact_with_info(const Tensor & self, bool pivot=true);
CAFFE2_API Tensor & btrisolve_out(Tensor & result, const Tensor & self, const Tensor & LU_data, const Tensor & LU_pivots);
CAFFE2_API Tensor btrisolve(const Tensor & self, const Tensor & LU_data, const Tensor & LU_pivots);
CAFFE2_API Tensor & multinomial_out(Tensor & result, const Tensor & self, int64_t num_samples, bool replacement=false, Generator * generator=nullptr);
CAFFE2_API Tensor multinomial(const Tensor & self, int64_t num_samples, bool replacement=false, Generator * generator=nullptr);
CAFFE2_API Tensor & lgamma_out(Tensor & result, const Tensor & self);
CAFFE2_API Tensor lgamma(const Tensor & self);
CAFFE2_API Tensor & digamma_out(Tensor & result, const Tensor & self);
CAFFE2_API Tensor digamma(const Tensor & self);
CAFFE2_API Tensor & polygamma_out(Tensor & result, int64_t n, const Tensor & self);
CAFFE2_API Tensor polygamma(int64_t n, const Tensor & self);
CAFFE2_API Tensor & erfinv_out(Tensor & result, const Tensor & self);
CAFFE2_API Tensor erfinv(const Tensor & self);
CAFFE2_API Tensor & frac_out(Tensor & result, const Tensor & self);
CAFFE2_API Tensor frac(const Tensor & self);
CAFFE2_API Tensor dist(const Tensor & self, const Tensor & other, Scalar p=2);
CAFFE2_API Tensor & reciprocal_out(Tensor & result, const Tensor & self);
CAFFE2_API Tensor reciprocal(const Tensor & self);
CAFFE2_API Tensor & neg_out(Tensor & result, const Tensor & self);
CAFFE2_API Tensor neg(const Tensor & self);
CAFFE2_API Tensor & atan2_out(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor atan2(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & lerp_out(Tensor & result, const Tensor & self, const Tensor & end, Scalar weight);
CAFFE2_API Tensor lerp(const Tensor & self, const Tensor & end, Scalar weight);
CAFFE2_API Tensor & histc_out(Tensor & result, const Tensor & self, int64_t bins=100, Scalar min=0, Scalar max=0);
CAFFE2_API Tensor histc(const Tensor & self, int64_t bins=100, Scalar min=0, Scalar max=0);
CAFFE2_API Tensor & sign_out(Tensor & result, const Tensor & self);
CAFFE2_API Tensor sign(const Tensor & self);
CAFFE2_API Tensor & fmod_out(Tensor & result, const Tensor & self, Scalar other);
CAFFE2_API Tensor fmod(const Tensor & self, Scalar other);
CAFFE2_API Tensor & fmod_out(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor fmod(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & remainder_out(Tensor & result, const Tensor & self, Scalar other);
CAFFE2_API Tensor remainder(const Tensor & self, Scalar other);
CAFFE2_API Tensor & remainder_out(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor remainder(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & min_out(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor min(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor min(const Tensor & self);
CAFFE2_API Tensor & max_out(Tensor & result, const Tensor & self, const Tensor & other);
CAFFE2_API Tensor max(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor max(const Tensor & self);
CAFFE2_API Tensor median(const Tensor & self);
CAFFE2_API std::tuple<Tensor &,Tensor &> sort_out(Tensor & values, Tensor & indices, const Tensor & self, int64_t dim=-1, bool descending=false);
CAFFE2_API std::tuple<Tensor,Tensor> sort(const Tensor & self, int64_t dim=-1, bool descending=false);
CAFFE2_API std::tuple<Tensor &,Tensor &> topk_out(Tensor & values, Tensor & indices, const Tensor & self, int64_t k, int64_t dim=-1, bool largest=true, bool sorted=true);
CAFFE2_API std::tuple<Tensor,Tensor> topk(const Tensor & self, int64_t k, int64_t dim=-1, bool largest=true, bool sorted=true);
CAFFE2_API Tensor all(const Tensor & self);
CAFFE2_API Tensor any(const Tensor & self);
CAFFE2_API Tensor & renorm_out(Tensor & result, const Tensor & self, Scalar p, int64_t dim, Scalar maxnorm);
CAFFE2_API Tensor renorm(const Tensor & self, Scalar p, int64_t dim, Scalar maxnorm);
CAFFE2_API Tensor unfold(const Tensor & self, int64_t dimension, int64_t size, int64_t step);
CAFFE2_API bool equal(const Tensor & self, const Tensor & other);
CAFFE2_API Tensor & pow_out(Tensor & result, const Tensor & self, const Tensor & exponent);
CAFFE2_API Tensor pow(const Tensor & self, const Tensor & exponent);
CAFFE2_API Tensor & pow_out(Tensor & result, Scalar self, const Tensor & exponent);
CAFFE2_API Tensor pow(Scalar self, const Tensor & exponent);
CAFFE2_API Tensor & normal_out(Tensor & output, const Tensor & mean, double std=1, Generator * generator=nullptr);
CAFFE2_API Tensor normal(const Tensor & mean, double std=1, Generator * generator=nullptr);
CAFFE2_API Tensor & normal_out(Tensor & output, double mean, const Tensor & std, Generator * generator=nullptr);
CAFFE2_API Tensor normal(double mean, const Tensor & std, Generator * generator=nullptr);
CAFFE2_API Tensor & normal_out(Tensor & output, const Tensor & mean, const Tensor & std, Generator * generator=nullptr);
CAFFE2_API Tensor normal(const Tensor & mean, const Tensor & std, Generator * generator=nullptr);
CAFFE2_API Tensor alias(const Tensor & self);
CAFFE2_API Tensor & _dirichlet_grad_out(Tensor & output, const Tensor & x, const Tensor & alpha, const Tensor & total);
CAFFE2_API Tensor _dirichlet_grad(const Tensor & x, const Tensor & alpha, const Tensor & total);
CAFFE2_API Tensor & binary_cross_entropy_out(Tensor & output, const Tensor & self, const Tensor & target, const Tensor & weight={}, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor binary_cross_entropy(const Tensor & self, const Tensor & target, const Tensor & weight={}, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor & binary_cross_entropy_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & target, const Tensor & weight, int64_t reduction);
CAFFE2_API Tensor binary_cross_entropy_backward(const Tensor & grad_output, const Tensor & self, const Tensor & target, const Tensor & weight, int64_t reduction);
CAFFE2_API Tensor & mse_loss_out(Tensor & output, const Tensor & self, const Tensor & target, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor mse_loss(const Tensor & self, const Tensor & target, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor & mse_loss_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & target, int64_t reduction);
CAFFE2_API Tensor mse_loss_backward(const Tensor & grad_output, const Tensor & self, const Tensor & target, int64_t reduction);
CAFFE2_API Tensor & l1_loss_out(Tensor & output, const Tensor & self, const Tensor & target, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor l1_loss(const Tensor & self, const Tensor & target, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor & l1_loss_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & target, int64_t reduction);
CAFFE2_API Tensor l1_loss_backward(const Tensor & grad_output, const Tensor & self, const Tensor & target, int64_t reduction);
CAFFE2_API Tensor & multi_margin_loss_out(Tensor & output, const Tensor & self, const Tensor & target, Scalar p=1, Scalar margin=1, const Tensor & weight={}, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor multi_margin_loss(const Tensor & self, const Tensor & target, Scalar p=1, Scalar margin=1, const Tensor & weight={}, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor & multi_margin_loss_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & target, Scalar p, Scalar margin, const Tensor & weight, int64_t reduction);
CAFFE2_API Tensor multi_margin_loss_backward(const Tensor & grad_output, const Tensor & self, const Tensor & target, Scalar p, Scalar margin, const Tensor & weight, int64_t reduction);
CAFFE2_API Tensor & multilabel_margin_loss_out(Tensor & output, const Tensor & self, const Tensor & target, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor multilabel_margin_loss(const Tensor & self, const Tensor & target, int64_t reduction=Reduction::Mean);
CAFFE2_API std::tuple<Tensor &,Tensor &> multilabel_margin_loss_forward_out(Tensor & output, Tensor & is_target, const Tensor & self, const Tensor & target, int64_t reduction);
CAFFE2_API std::tuple<Tensor,Tensor> multilabel_margin_loss_forward(const Tensor & self, const Tensor & target, int64_t reduction);
CAFFE2_API Tensor & multilabel_margin_loss_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & target, int64_t reduction, const Tensor & is_target);
CAFFE2_API Tensor multilabel_margin_loss_backward(const Tensor & grad_output, const Tensor & self, const Tensor & target, int64_t reduction, const Tensor & is_target);
CAFFE2_API Tensor & nll_loss_out(Tensor & output, const Tensor & self, const Tensor & target, const Tensor & weight={}, int64_t reduction=Reduction::Mean, int64_t ignore_index=-100);
CAFFE2_API Tensor nll_loss(const Tensor & self, const Tensor & target, const Tensor & weight={}, int64_t reduction=Reduction::Mean, int64_t ignore_index=-100);
CAFFE2_API std::tuple<Tensor &,Tensor &> nll_loss_forward_out(Tensor & output, Tensor & total_weight, const Tensor & self, const Tensor & target, const Tensor & weight, int64_t reduction, int64_t ignore_index);
CAFFE2_API std::tuple<Tensor,Tensor> nll_loss_forward(const Tensor & self, const Tensor & target, const Tensor & weight, int64_t reduction, int64_t ignore_index);
CAFFE2_API Tensor & nll_loss_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & target, const Tensor & weight, int64_t reduction, int64_t ignore_index, const Tensor & total_weight);
CAFFE2_API Tensor nll_loss_backward(const Tensor & grad_output, const Tensor & self, const Tensor & target, const Tensor & weight, int64_t reduction, int64_t ignore_index, const Tensor & total_weight);
CAFFE2_API Tensor & nll_loss2d_out(Tensor & output, const Tensor & self, const Tensor & target, const Tensor & weight={}, int64_t reduction=Reduction::Mean, int64_t ignore_index=-100);
CAFFE2_API Tensor nll_loss2d(const Tensor & self, const Tensor & target, const Tensor & weight={}, int64_t reduction=Reduction::Mean, int64_t ignore_index=-100);
CAFFE2_API std::tuple<Tensor &,Tensor &> nll_loss2d_forward_out(Tensor & output, Tensor & total_weight, const Tensor & self, const Tensor & target, const Tensor & weight, int64_t reduction, int64_t ignore_index);
CAFFE2_API std::tuple<Tensor,Tensor> nll_loss2d_forward(const Tensor & self, const Tensor & target, const Tensor & weight, int64_t reduction, int64_t ignore_index);
CAFFE2_API Tensor & nll_loss2d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & target, const Tensor & weight, int64_t reduction, int64_t ignore_index, const Tensor & total_weight);
CAFFE2_API Tensor nll_loss2d_backward(const Tensor & grad_output, const Tensor & self, const Tensor & target, const Tensor & weight, int64_t reduction, int64_t ignore_index, const Tensor & total_weight);
CAFFE2_API Tensor & smooth_l1_loss_out(Tensor & output, const Tensor & self, const Tensor & target, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor smooth_l1_loss(const Tensor & self, const Tensor & target, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor & smooth_l1_loss_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & target, int64_t reduction);
CAFFE2_API Tensor smooth_l1_loss_backward(const Tensor & grad_output, const Tensor & self, const Tensor & target, int64_t reduction);
CAFFE2_API Tensor & soft_margin_loss_out(Tensor & output, const Tensor & self, const Tensor & target, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor soft_margin_loss(const Tensor & self, const Tensor & target, int64_t reduction=Reduction::Mean);
CAFFE2_API Tensor & soft_margin_loss_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & target, int64_t reduction);
CAFFE2_API Tensor soft_margin_loss_backward(const Tensor & grad_output, const Tensor & self, const Tensor & target, int64_t reduction);
CAFFE2_API Tensor & elu_out(Tensor & output, const Tensor & self, Scalar alpha=1, Scalar scale=1, Scalar input_scale=1);
CAFFE2_API Tensor elu(const Tensor & self, Scalar alpha=1, Scalar scale=1, Scalar input_scale=1);
CAFFE2_API Tensor & elu_backward_out(Tensor & grad_input, const Tensor & grad_output, Scalar alpha, Scalar scale, Scalar input_scale, const Tensor & output);
CAFFE2_API Tensor elu_backward(const Tensor & grad_output, Scalar alpha, Scalar scale, Scalar input_scale, const Tensor & output);
CAFFE2_API Tensor & elu_(Tensor & self, Scalar alpha=1, Scalar scale=1, Scalar input_scale=1);
CAFFE2_API Tensor & glu_out(Tensor & output, const Tensor & self, int64_t dim=-1);
CAFFE2_API Tensor glu(const Tensor & self, int64_t dim=-1);
CAFFE2_API Tensor & glu_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, int64_t dim);
CAFFE2_API Tensor glu_backward(const Tensor & grad_output, const Tensor & self, int64_t dim);
CAFFE2_API Tensor & hardtanh_out(Tensor & output, const Tensor & self, Scalar min_val=-1, Scalar max_val=1);
CAFFE2_API Tensor hardtanh(const Tensor & self, Scalar min_val=-1, Scalar max_val=1);
CAFFE2_API Tensor & hardtanh_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, Scalar min_val, Scalar max_val);
CAFFE2_API Tensor hardtanh_backward(const Tensor & grad_output, const Tensor & self, Scalar min_val, Scalar max_val);
CAFFE2_API Tensor & hardtanh_(Tensor & self, Scalar min_val=-1, Scalar max_val=1);
CAFFE2_API Tensor & leaky_relu_out(Tensor & output, const Tensor & self, Scalar negative_slope=0.01);
CAFFE2_API Tensor leaky_relu(const Tensor & self, Scalar negative_slope=0.01);
CAFFE2_API Tensor & leaky_relu_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, Scalar negative_slope);
CAFFE2_API Tensor leaky_relu_backward(const Tensor & grad_output, const Tensor & self, Scalar negative_slope);
CAFFE2_API Tensor & leaky_relu_(Tensor & self, Scalar negative_slope=0.01);
CAFFE2_API Tensor & log_sigmoid_out(Tensor & output, const Tensor & self);
CAFFE2_API Tensor log_sigmoid(const Tensor & self);
CAFFE2_API std::tuple<Tensor &,Tensor &> log_sigmoid_forward_out(Tensor & output, Tensor & buffer, const Tensor & self);
CAFFE2_API std::tuple<Tensor,Tensor> log_sigmoid_forward(const Tensor & self);
CAFFE2_API Tensor & log_sigmoid_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & buffer);
CAFFE2_API Tensor log_sigmoid_backward(const Tensor & grad_output, const Tensor & self, const Tensor & buffer);
CAFFE2_API Tensor & rrelu_with_noise_out(Tensor & output, const Tensor & self, const Tensor & noise, Scalar lower=0.125, Scalar upper=0.3333333333333333, bool training=false, Generator * generator=nullptr);
CAFFE2_API Tensor rrelu_with_noise(const Tensor & self, const Tensor & noise, Scalar lower=0.125, Scalar upper=0.3333333333333333, bool training=false, Generator * generator=nullptr);
CAFFE2_API Tensor & rrelu_with_noise_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & noise, Scalar lower, Scalar upper, bool training);
CAFFE2_API Tensor rrelu_with_noise_backward(const Tensor & grad_output, const Tensor & self, const Tensor & noise, Scalar lower, Scalar upper, bool training);
CAFFE2_API Tensor & rrelu_with_noise_(Tensor & self, const Tensor & noise, Scalar lower=0.125, Scalar upper=0.3333333333333333, bool training=false, Generator * generator=nullptr);
CAFFE2_API Tensor & softplus_out(Tensor & output, const Tensor & self, Scalar beta=1, Scalar threshold=20);
CAFFE2_API Tensor softplus(const Tensor & self, Scalar beta=1, Scalar threshold=20);
CAFFE2_API Tensor & softplus_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, Scalar beta, Scalar threshold, const Tensor & output);
CAFFE2_API Tensor softplus_backward(const Tensor & grad_output, const Tensor & self, Scalar beta, Scalar threshold, const Tensor & output);
CAFFE2_API Tensor & softshrink_out(Tensor & output, const Tensor & self, Scalar lambd=0.5);
CAFFE2_API Tensor softshrink(const Tensor & self, Scalar lambd=0.5);
CAFFE2_API Tensor & softshrink_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, Scalar lambd);
CAFFE2_API Tensor softshrink_backward(const Tensor & grad_output, const Tensor & self, Scalar lambd);
CAFFE2_API Tensor & adaptive_avg_pool2d_out(Tensor & output, const Tensor & self, IntList output_size);
CAFFE2_API Tensor adaptive_avg_pool2d(const Tensor & self, IntList output_size);
CAFFE2_API Tensor & adaptive_avg_pool2d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self);
CAFFE2_API Tensor adaptive_avg_pool2d_backward(const Tensor & grad_output, const Tensor & self);
CAFFE2_API Tensor & adaptive_avg_pool3d_out(Tensor & output, const Tensor & self, IntList output_size);
CAFFE2_API Tensor adaptive_avg_pool3d(const Tensor & self, IntList output_size);
CAFFE2_API Tensor & adaptive_avg_pool3d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self);
CAFFE2_API Tensor adaptive_avg_pool3d_backward(const Tensor & grad_output, const Tensor & self);
CAFFE2_API std::tuple<Tensor &,Tensor &> adaptive_max_pool2d_out(Tensor & output, Tensor & indices, const Tensor & self, IntList output_size);
CAFFE2_API std::tuple<Tensor,Tensor> adaptive_max_pool2d(const Tensor & self, IntList output_size);
CAFFE2_API Tensor & adaptive_max_pool2d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & indices);
CAFFE2_API Tensor adaptive_max_pool2d_backward(const Tensor & grad_output, const Tensor & self, const Tensor & indices);
CAFFE2_API std::tuple<Tensor &,Tensor &> adaptive_max_pool3d_out(Tensor & output, Tensor & indices, const Tensor & self, IntList output_size);
CAFFE2_API std::tuple<Tensor,Tensor> adaptive_max_pool3d(const Tensor & self, IntList output_size);
CAFFE2_API Tensor & adaptive_max_pool3d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & indices);
CAFFE2_API Tensor adaptive_max_pool3d_backward(const Tensor & grad_output, const Tensor & self, const Tensor & indices);
CAFFE2_API Tensor & avg_pool2d_out(Tensor & output, const Tensor & self, IntList kernel_size, IntList stride={}, IntList padding=0, bool ceil_mode=false, bool count_include_pad=true);
CAFFE2_API Tensor avg_pool2d(const Tensor & self, IntList kernel_size, IntList stride={}, IntList padding=0, bool ceil_mode=false, bool count_include_pad=true);
CAFFE2_API Tensor & avg_pool2d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, IntList kernel_size, IntList stride, IntList padding, bool ceil_mode, bool count_include_pad);
CAFFE2_API Tensor avg_pool2d_backward(const Tensor & grad_output, const Tensor & self, IntList kernel_size, IntList stride, IntList padding, bool ceil_mode, bool count_include_pad);
CAFFE2_API Tensor & avg_pool3d_out(Tensor & output, const Tensor & self, IntList kernel_size, IntList stride={}, IntList padding=0, bool ceil_mode=false, bool count_include_pad=true);
CAFFE2_API Tensor avg_pool3d(const Tensor & self, IntList kernel_size, IntList stride={}, IntList padding=0, bool ceil_mode=false, bool count_include_pad=true);
CAFFE2_API Tensor & avg_pool3d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, IntList kernel_size, IntList stride, IntList padding, bool ceil_mode, bool count_include_pad);
CAFFE2_API Tensor avg_pool3d_backward(const Tensor & grad_output, const Tensor & self, IntList kernel_size, IntList stride, IntList padding, bool ceil_mode, bool count_include_pad);
CAFFE2_API std::tuple<Tensor &,Tensor &> fractional_max_pool2d_out(Tensor & output, Tensor & indices, const Tensor & self, IntList kernel_size, IntList output_size, const Tensor & random_samples);
CAFFE2_API std::tuple<Tensor,Tensor> fractional_max_pool2d(const Tensor & self, IntList kernel_size, IntList output_size, const Tensor & random_samples);
CAFFE2_API Tensor & fractional_max_pool2d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, IntList kernel_size, IntList output_size, const Tensor & indices);
CAFFE2_API Tensor fractional_max_pool2d_backward(const Tensor & grad_output, const Tensor & self, IntList kernel_size, IntList output_size, const Tensor & indices);
CAFFE2_API std::tuple<Tensor &,Tensor &> max_pool2d_with_indices_out(Tensor & output, Tensor & indices, const Tensor & self, IntList kernel_size, IntList stride={}, IntList padding=0, IntList dilation=1, bool ceil_mode=false);
CAFFE2_API std::tuple<Tensor,Tensor> max_pool2d_with_indices(const Tensor & self, IntList kernel_size, IntList stride={}, IntList padding=0, IntList dilation=1, bool ceil_mode=false);
CAFFE2_API Tensor & max_pool2d_with_indices_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, IntList kernel_size, IntList stride, IntList padding, IntList dilation, bool ceil_mode, const Tensor & indices);
CAFFE2_API Tensor max_pool2d_with_indices_backward(const Tensor & grad_output, const Tensor & self, IntList kernel_size, IntList stride, IntList padding, IntList dilation, bool ceil_mode, const Tensor & indices);
CAFFE2_API std::tuple<Tensor &,Tensor &> max_pool3d_with_indices_out(Tensor & output, Tensor & indices, const Tensor & self, IntList kernel_size, IntList stride={}, IntList padding=0, IntList dilation=1, bool ceil_mode=false);
CAFFE2_API std::tuple<Tensor,Tensor> max_pool3d_with_indices(const Tensor & self, IntList kernel_size, IntList stride={}, IntList padding=0, IntList dilation=1, bool ceil_mode=false);
CAFFE2_API Tensor & max_pool3d_with_indices_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, IntList kernel_size, IntList stride, IntList padding, IntList dilation, bool ceil_mode, const Tensor & indices);
CAFFE2_API Tensor max_pool3d_with_indices_backward(const Tensor & grad_output, const Tensor & self, IntList kernel_size, IntList stride, IntList padding, IntList dilation, bool ceil_mode, const Tensor & indices);
CAFFE2_API Tensor & max_unpool2d_out(Tensor & output, const Tensor & self, const Tensor & indices, IntList output_size);
CAFFE2_API Tensor max_unpool2d(const Tensor & self, const Tensor & indices, IntList output_size);
CAFFE2_API Tensor & max_unpool2d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & indices, IntList output_size);
CAFFE2_API Tensor max_unpool2d_backward(const Tensor & grad_output, const Tensor & self, const Tensor & indices, IntList output_size);
CAFFE2_API Tensor & max_unpool3d_out(Tensor & output, const Tensor & self, const Tensor & indices, IntList output_size, IntList stride, IntList padding);
CAFFE2_API Tensor max_unpool3d(const Tensor & self, const Tensor & indices, IntList output_size, IntList stride, IntList padding);
CAFFE2_API Tensor & max_unpool3d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, const Tensor & indices, IntList output_size, IntList stride, IntList padding);
CAFFE2_API Tensor max_unpool3d_backward(const Tensor & grad_output, const Tensor & self, const Tensor & indices, IntList output_size, IntList stride, IntList padding);
CAFFE2_API Tensor & reflection_pad1d_out(Tensor & output, const Tensor & self, IntList padding);
CAFFE2_API Tensor reflection_pad1d(const Tensor & self, IntList padding);
CAFFE2_API Tensor & reflection_pad1d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, IntList padding);
CAFFE2_API Tensor reflection_pad1d_backward(const Tensor & grad_output, const Tensor & self, IntList padding);
CAFFE2_API Tensor & reflection_pad2d_out(Tensor & output, const Tensor & self, IntList padding);
CAFFE2_API Tensor reflection_pad2d(const Tensor & self, IntList padding);
CAFFE2_API Tensor & reflection_pad2d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, IntList padding);
CAFFE2_API Tensor reflection_pad2d_backward(const Tensor & grad_output, const Tensor & self, IntList padding);
CAFFE2_API Tensor & replication_pad1d_out(Tensor & output, const Tensor & self, IntList padding);
CAFFE2_API Tensor replication_pad1d(const Tensor & self, IntList padding);
CAFFE2_API Tensor & replication_pad1d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, IntList padding);
CAFFE2_API Tensor replication_pad1d_backward(const Tensor & grad_output, const Tensor & self, IntList padding);
CAFFE2_API Tensor & replication_pad2d_out(Tensor & output, const Tensor & self, IntList padding);
CAFFE2_API Tensor replication_pad2d(const Tensor & self, IntList padding);
CAFFE2_API Tensor & replication_pad2d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, IntList padding);
CAFFE2_API Tensor replication_pad2d_backward(const Tensor & grad_output, const Tensor & self, IntList padding);
CAFFE2_API Tensor & replication_pad3d_out(Tensor & output, const Tensor & self, IntList padding);
CAFFE2_API Tensor replication_pad3d(const Tensor & self, IntList padding);
CAFFE2_API Tensor & replication_pad3d_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & self, IntList padding);
CAFFE2_API Tensor replication_pad3d_backward(const Tensor & grad_output, const Tensor & self, IntList padding);
CAFFE2_API Tensor & upsample_linear1d_out(Tensor & output, const Tensor & self, IntList output_size, bool align_corners);
CAFFE2_API Tensor upsample_linear1d(const Tensor & self, IntList output_size, bool align_corners);
CAFFE2_API Tensor & upsample_linear1d_backward_out(Tensor & grad_input, const Tensor & grad_output, IntList output_size, IntList input_size, bool align_corners);
CAFFE2_API Tensor upsample_linear1d_backward(const Tensor & grad_output, IntList output_size, IntList input_size, bool align_corners);
CAFFE2_API Tensor & upsample_bilinear2d_out(Tensor & output, const Tensor & self, IntList output_size, bool align_corners);
CAFFE2_API Tensor upsample_bilinear2d(const Tensor & self, IntList output_size, bool align_corners);
CAFFE2_API Tensor & upsample_bilinear2d_backward_out(Tensor & grad_input, const Tensor & grad_output, IntList output_size, IntList input_size, bool align_corners);
CAFFE2_API Tensor upsample_bilinear2d_backward(const Tensor & grad_output, IntList output_size, IntList input_size, bool align_corners);
CAFFE2_API Tensor & upsample_trilinear3d_out(Tensor & output, const Tensor & self, IntList output_size, bool align_corners);
CAFFE2_API Tensor upsample_trilinear3d(const Tensor & self, IntList output_size, bool align_corners);
CAFFE2_API Tensor & upsample_trilinear3d_backward_out(Tensor & grad_input, const Tensor & grad_output, IntList output_size, IntList input_size, bool align_corners);
CAFFE2_API Tensor upsample_trilinear3d_backward(const Tensor & grad_output, IntList output_size, IntList input_size, bool align_corners);
CAFFE2_API Tensor & upsample_nearest1d_out(Tensor & output, const Tensor & self, IntList output_size);
CAFFE2_API Tensor upsample_nearest1d(const Tensor & self, IntList output_size);
CAFFE2_API Tensor & upsample_nearest1d_backward_out(Tensor & grad_input, const Tensor & grad_output, IntList output_size, IntList input_size);
CAFFE2_API Tensor upsample_nearest1d_backward(const Tensor & grad_output, IntList output_size, IntList input_size);
CAFFE2_API Tensor & upsample_nearest2d_out(Tensor & output, const Tensor & self, IntList output_size);
CAFFE2_API Tensor upsample_nearest2d(const Tensor & self, IntList output_size);
CAFFE2_API Tensor & upsample_nearest2d_backward_out(Tensor & grad_input, const Tensor & grad_output, IntList output_size, IntList input_size);
CAFFE2_API Tensor upsample_nearest2d_backward(const Tensor & grad_output, IntList output_size, IntList input_size);
CAFFE2_API Tensor & upsample_nearest3d_out(Tensor & output, const Tensor & self, IntList output_size);
CAFFE2_API Tensor upsample_nearest3d(const Tensor & self, IntList output_size);
CAFFE2_API Tensor & upsample_nearest3d_backward_out(Tensor & grad_input, const Tensor & grad_output, IntList output_size, IntList input_size);
CAFFE2_API Tensor upsample_nearest3d_backward(const Tensor & grad_output, IntList output_size, IntList input_size);
CAFFE2_API Tensor & sigmoid_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & output);
CAFFE2_API Tensor sigmoid_backward(const Tensor & grad_output, const Tensor & output);
CAFFE2_API Tensor & tanh_backward_out(Tensor & grad_input, const Tensor & grad_output, const Tensor & output);
CAFFE2_API Tensor tanh_backward(const Tensor & grad_output, const Tensor & output);
CAFFE2_API Tensor & thnn_conv_transpose2d_out(Tensor & output, const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList output_padding=0, IntList dilation=1);
CAFFE2_API Tensor thnn_conv_transpose2d(const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList output_padding=0, IntList dilation=1);
CAFFE2_API std::tuple<Tensor &,Tensor &,Tensor &> thnn_conv_transpose2d_forward_out(Tensor & output, Tensor & columns, Tensor & ones, const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias, IntList stride, IntList padding, IntList output_padding, IntList dilation);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> thnn_conv_transpose2d_forward(const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias, IntList stride, IntList padding, IntList output_padding, IntList dilation);
CAFFE2_API std::tuple<Tensor &,Tensor &,Tensor &> thnn_conv_transpose2d_backward_out(Tensor & grad_input, Tensor & grad_weight, Tensor & grad_bias, const Tensor & grad_output, const Tensor & self, const Tensor & weight, IntList kernel_size, IntList stride, IntList padding, IntList output_padding, IntList dilation, const Tensor & columns, const Tensor & ones);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> thnn_conv_transpose2d_backward(const Tensor & grad_output, const Tensor & self, const Tensor & weight, IntList kernel_size, IntList stride, IntList padding, IntList output_padding, IntList dilation, const Tensor & columns, const Tensor & ones, std::array<bool,3> output_mask);
CAFFE2_API Tensor & thnn_conv_transpose3d_out(Tensor & output, const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList output_padding=0, IntList dilation=1);
CAFFE2_API Tensor thnn_conv_transpose3d(const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList output_padding=0, IntList dilation=1);
CAFFE2_API std::tuple<Tensor &,Tensor &,Tensor &> thnn_conv_transpose3d_forward_out(Tensor & output, Tensor & finput, Tensor & fgrad_input, const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias, IntList stride, IntList padding, IntList output_padding, IntList dilation);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> thnn_conv_transpose3d_forward(const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias, IntList stride, IntList padding, IntList output_padding, IntList dilation);
CAFFE2_API std::tuple<Tensor &,Tensor &,Tensor &> thnn_conv_transpose3d_backward_out(Tensor & grad_input, Tensor & grad_weight, Tensor & grad_bias, const Tensor & grad_output, const Tensor & self, const Tensor & weight, IntList kernel_size, IntList stride, IntList padding, IntList output_padding, IntList dilation, const Tensor & finput, const Tensor & fgrad_input);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> thnn_conv_transpose3d_backward(const Tensor & grad_output, const Tensor & self, const Tensor & weight, IntList kernel_size, IntList stride, IntList padding, IntList output_padding, IntList dilation, const Tensor & finput, const Tensor & fgrad_input, std::array<bool,3> output_mask);
CAFFE2_API Tensor & thnn_conv2d_out(Tensor & output, const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias={}, IntList stride=1, IntList padding=0);
CAFFE2_API Tensor thnn_conv2d(const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias={}, IntList stride=1, IntList padding=0);
CAFFE2_API std::tuple<Tensor &,Tensor &,Tensor &> thnn_conv2d_forward_out(Tensor & output, Tensor & finput, Tensor & fgrad_input, const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias, IntList stride, IntList padding);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> thnn_conv2d_forward(const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias, IntList stride, IntList padding);
CAFFE2_API std::tuple<Tensor &,Tensor &,Tensor &> thnn_conv2d_backward_out(Tensor & grad_input, Tensor & grad_weight, Tensor & grad_bias, const Tensor & grad_output, const Tensor & self, const Tensor & weight, IntList kernel_size, IntList stride, IntList padding, const Tensor & finput, const Tensor & fgrad_input);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> thnn_conv2d_backward(const Tensor & grad_output, const Tensor & self, const Tensor & weight, IntList kernel_size, IntList stride, IntList padding, const Tensor & finput, const Tensor & fgrad_input, std::array<bool,3> output_mask);
CAFFE2_API Tensor & thnn_conv_depthwise2d_out(Tensor & output, const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList dilation=1);
CAFFE2_API Tensor thnn_conv_depthwise2d(const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList dilation=1);
CAFFE2_API Tensor & thnn_conv_depthwise2d_forward_out(Tensor & output, const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias, IntList stride, IntList padding, IntList dilation);
CAFFE2_API Tensor thnn_conv_depthwise2d_forward(const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias, IntList stride, IntList padding, IntList dilation);
CAFFE2_API std::tuple<Tensor &,Tensor &> thnn_conv_depthwise2d_backward_out(Tensor & grad_input, Tensor & grad_weight, const Tensor & grad_output, const Tensor & self, const Tensor & weight, IntList kernel_size, IntList stride, IntList padding, IntList dilation);
CAFFE2_API std::tuple<Tensor,Tensor> thnn_conv_depthwise2d_backward(const Tensor & grad_output, const Tensor & self, const Tensor & weight, IntList kernel_size, IntList stride, IntList padding, IntList dilation, std::array<bool,2> output_mask);
CAFFE2_API Tensor & thnn_conv3d_out(Tensor & output, const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias={}, IntList stride=1, IntList padding=0);
CAFFE2_API Tensor thnn_conv3d(const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias={}, IntList stride=1, IntList padding=0);
CAFFE2_API std::tuple<Tensor &,Tensor &,Tensor &> thnn_conv3d_forward_out(Tensor & output, Tensor & finput, Tensor & fgrad_input, const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias, IntList stride, IntList padding);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> thnn_conv3d_forward(const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias, IntList stride, IntList padding);
CAFFE2_API std::tuple<Tensor &,Tensor &,Tensor &> thnn_conv3d_backward_out(Tensor & grad_input, Tensor & grad_weight, Tensor & grad_bias, const Tensor & grad_output, const Tensor & self, const Tensor & weight, IntList kernel_size, IntList stride, IntList padding, const Tensor & finput, const Tensor & fgrad_input);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> thnn_conv3d_backward(const Tensor & grad_output, const Tensor & self, const Tensor & weight, IntList kernel_size, IntList stride, IntList padding, const Tensor & finput, const Tensor & fgrad_input, std::array<bool,3> output_mask);
CAFFE2_API Tensor & thnn_conv_dilated2d_out(Tensor & output, const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList dilation=1);
CAFFE2_API Tensor thnn_conv_dilated2d(const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList dilation=1);
CAFFE2_API std::tuple<Tensor &,Tensor &,Tensor &> thnn_conv_dilated2d_forward_out(Tensor & output, Tensor & columns, Tensor & ones, const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias, IntList stride, IntList padding, IntList dilation);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> thnn_conv_dilated2d_forward(const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias, IntList stride, IntList padding, IntList dilation);
CAFFE2_API std::tuple<Tensor &,Tensor &,Tensor &> thnn_conv_dilated2d_backward_out(Tensor & grad_input, Tensor & grad_weight, Tensor & grad_bias, const Tensor & grad_output, const Tensor & self, const Tensor & weight, IntList kernel_size, IntList stride, IntList padding, IntList dilation, const Tensor & columns, const Tensor & ones);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> thnn_conv_dilated2d_backward(const Tensor & grad_output, const Tensor & self, const Tensor & weight, IntList kernel_size, IntList stride, IntList padding, IntList dilation, const Tensor & columns, const Tensor & ones, std::array<bool,3> output_mask);
CAFFE2_API Tensor & thnn_conv_dilated3d_out(Tensor & output, const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList dilation=1);
CAFFE2_API Tensor thnn_conv_dilated3d(const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias={}, IntList stride=1, IntList padding=0, IntList dilation=1);
CAFFE2_API std::tuple<Tensor &,Tensor &,Tensor &> thnn_conv_dilated3d_forward_out(Tensor & output, Tensor & columns, Tensor & ones, const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias, IntList stride, IntList padding, IntList dilation);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> thnn_conv_dilated3d_forward(const Tensor & self, const Tensor & weight, IntList kernel_size, const Tensor & bias, IntList stride, IntList padding, IntList dilation);
CAFFE2_API std::tuple<Tensor &,Tensor &,Tensor &> thnn_conv_dilated3d_backward_out(Tensor & grad_input, Tensor & grad_weight, Tensor & grad_bias, const Tensor & grad_output, const Tensor & self, const Tensor & weight, IntList kernel_size, IntList stride, IntList padding, IntList dilation, const Tensor & columns, const Tensor & ones);
CAFFE2_API std::tuple<Tensor,Tensor,Tensor> thnn_conv_dilated3d_backward(const Tensor & grad_output, const Tensor & self, const Tensor & weight, IntList kernel_size, IntList stride, IntList padding, IntList dilation, const Tensor & columns, const Tensor & ones, std::array<bool,3> output_mask);

} // namespace native
} // namespace at
